/* 20bsp.cdf - BSP-specific component descriptor file */

/*
 * Copyright (c) 2022 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

Selection BOARD_SELECTION {
    NAME        Board Selection
    COUNT       1-1
    CHILDREN    INCLUDE_INTEL_AGILEX5_GSRD
    DEFAULTS    INCLUDE_INTEL_AGILEX5_GSRD
    _CHILDREN   FOLDER_HIDDEN
}

Component INCLUDE_INTEL_AGILEX5_GSRD {
    NAME        Intel Agilex SoC Development Kit
    SYNOPSIS    This component is used to select the configuration of \
                Intel Agilex SoC GSRD.
    REQUIRES    DRV_INTCTLR_FDT_ARM_GIC
}

Parameter DTS_FILE {
    NAME        DTS file name
    SYNOPSIS    This parameter defines the device tree source (DTS) file name to be used.
    DEFAULT     (INCLUDE_INTEL_AGILEX5_GSRD)::(socfpga_agilex5_socdk.dts)
}

Parameter DEFAULT_BOOT_LINE {
    NAME        Default boot line
    SYNOPSIS    This parameter specifies the default boot line string. \
                Used when it is not defined in dts file.
    TYPE        string
    DEFAULT     "emac(0,0)host:vxWorks h=192.168.0.2 e=192.168.0.3:ffffff00 u=target pw=vxTarget"
}

Profile BSP_DEFAULT {
    PROFILES   PROFILE_OS_DEFAULT
    COMPONENTS +=  \
        DRV_SIO_FDT_NS16550 \
        DRV_ARM_GEN_TIMER \
        DRV_TIMER_FDT_SYNOPSYS_DW_TIMER \
        DRV_MCP_SDHC_CTRL \
        DRV_SDSTORAGE_CARD \
        DRV_END_FDT_DWXGMAC \
        INCLUDE_MARVELL_PHY \
        DRV_SYNOPSYS_DW_I2C \
        DRV_I2C_EEPROM  \
        DRV_FDT_SYNOPSYS_DW_GPIO \
        INCLUDE_SYNOPSYSHCI_INIT \
        DRV_USB_PHY_NOP \
        INCLUDE_USB_DWC3_HCD \
        DRV_VXB_HPNFC \
        INCLUDE_MTD \
        INCLUDE_MTD_SHOW \
        INCLUDE_FTL \
        INCLUDE_FTL_FORMAT \
        INCLUDE_FTL_SHOW \
        DRV_FDT_ALT5QSPIFLASH \
        DRV_SPI_FLASH \
        INCLUDE_MTD_VXBFLASH \
        INCLUDE_TFFS \
        INCLUDE_TL_FTL
}

Parameter CONSOLE_BAUD_RATE {
    NAME        Baud rate of console port
    DEFAULT     115200
}

/*
* VX_SMP_NUM_CPUS is a SMP parameter only and only available for SMP
* builds. Due to a limitation of the project tool at the time this
* parameter is created where the tool can not recognize the ifdef SMP
* selection, this parameter is set up such that _CFG_PARAMS is not
* specified here. In the 00vxWorks.cdf file, where the parameter
* VX_SMP_NUM_CPUS is defined, the _CFG_PARAMS is specified only for
* VxWorks SMP. Hence the redefining of VX_SMP_NUM_CPUS here should only
* override the value and not the rest of the properties. And for UP, the
* parameter is ignored since the parameter is not tied to any component
* (_CFG_PARAMS is not specified).
*/

Parameter VX_SMP_NUM_CPUS {
    NAME        Number of SMP-enabled CPUs
    SYNOPSIS    This parameter specifies the number of CPUs to be enabled for SMP.
    TYPE        UINT
    DEFAULT     (4)
}

/* 1st 32MB of DRAM is reserved */

Parameter LOCAL_MEM_PHYS_ADRS {
    NAME        Physical base address
    SYNOPSIS    This parameter specifies the local memory base physical address.
    DEFAULT     0x80000000
}

Parameter STATIC_MMU_TABLE_BASE {
    NAME        Static MMU table base address
    SYNOPSIS    This parameter defines the static MMU table base address for the early MMU.
    TYPE        uint
    DEFAULT     (0x82080000)
}

Parameter LOCAL_MEM_LOCAL_ADRS {
    NAME        System memory start address
    SYNOPSIS    This parameter specifies the local memory base virtual address.
#ifdef _WRS_CONFIG_LP64
    DEFAULT     0xffffffff80000000
#else
    DEFAULT     0x00000000
#endif
}

Parameter RAM_LOW_ADRS {
    NAME        Run-time kernel entry address
    SYNOPSIS    This parameter specifies the RAM address which is the entry \
                point of VxWorks.
#ifdef _WRS_CONFIG_LP64
    DEFAULT     (INCLUDE_BOOT_APP)::(0xffffffff81000000) \
                0xffffffff80100000
#else
    DEFAULT     (INCLUDE_BOOT_APP)::(0x01000000) \
                0x00100000
#endif
}

Parameter RAM_HIGH_ADRS {
    NAME        Run-time kernel high address
    SYNOPSIS    This parameter specifies the RAM address which is the entry \
                point of the boot application.
#ifdef _WRS_CONFIG_LP64
    DEFAULT     (INCLUDE_BOOT_APP)::(0xffffffff82000000) \
                0xffffffff81000000
#else
    DEFAULT     (INCLUDE_BOOT_APP)::(0x02000000) \
                0x01000000
#endif
}

Parameter KERNEL_LOAD_ADRS {
    NAME        Run-time kernel load address
    SYNOPSIS    This parameter specifies the physical address for the VxWorks \
                image entry point.
    DEFAULT     (INCLUDE_BOOT_APP)::(0x83000000) \
                0x82100000
}

Parameter DTB_RELOC_ADDR {
    NAME        Device tree blob (DTB) relocation address
    SYNOPSIS    The DTB must be relocated to a safe address so that it is not \
                overwritten. This address should be below RAM_LOW_ADRS and the \
                reserved start region. This address must allow sufficient space\
                for the entire DTB.
    TYPE        void *
    DEFAULT     (LOCAL_MEM_LOCAL_ADRS + 0x10000)
}

Parameter ISR_STACK_SIZE {
    NAME        ISR stack size
    SYNOPSIS    This parameter defines the ISR stack size, in bytes.
    DEFAULT     0x2000
}
