/* vxbFdtSynopsysDwGpio.h - GPIO driver hardware defintions */

/*
 * Copyright (c) 2014, 2016, 2018, 2021-2022 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */
#ifndef __INCvxbFdtSynopsysDwGpioh
#define __INCvxbFdtSynopsysDwGpioh

#ifdef __cplusplus
extern "C" {
#endif

/* defines */

#define DW_GPIO_DRIVER_NAME                    "designwareGpio"

#define DW_MAX_GPIO_BANK_WIDTH                 (32)
#define DW_GPIO_IRQ_SENSITIVE_NONE             (0x0)
#define DW_GPIO_IRQ_LOW_LEVEL_SENSITIVE        (0x1)
#define DW_GPIO_IRQ_HIGH_LEVEL_SENSITIVE       (0x2)
#define DW_GPIO_IRQ_RISING_EDGE_SENSITIVE      (0x4)
#define DW_GPIO_IRQ_FALLING_EDGE_SENSITIVE     (0x8)
#define DW_GPIO_VALUE_LOW                      (0)
#define DW_GPIO_VALUE_HIGH                     (1)
#define DW_GPIO_VALUE_INVALID                  (0xff)
#define DW_GPIO_PORTA                          0x00
#define DW_GPIO_PORTB                          0xff

/* register offsets */

#define SWPORT_DR_OFFSET(bk, bkShft, port)     (((bk) << (bkShft)) + \
                                                (0x00 + ((port) & 0x0c)))
#define SWPORT_DDR_OFFSET(bk,bkShft, port)     (((bk) << (bkShft)) + \
                                                (0x04 + ((port) & 0x0c)))
#define EXTPORT_IDR_OFFSET(bk, bkShft, port)   (((bk) << (bkShft)) + \
                                                (0x50 + ((port) & 0x04)))
#define INTEN_OFFSET(bk, bkShft)               (((bk) << (bkShft)) + 0x30)
#define INTMASK_OFFSET(bk, bkShft)             (((bk) << (bkShft)) + 0x34)
#define INTTYPELEVEL_OFFSET(bk, bkShft)        (((bk) << (bkShft)) + 0x38)
#define INTPOLARITY_OFFSET(bk, bkShft)         (((bk) << (bkShft)) + 0x3C)
#define INTSTATUS_OFFSET(bk, bkShft)           (((bk) << (bkShft)) + 0x40)
#define RAWINTSTATUS_OFFSET(bk, bkShft)        (((bk) << (bkShft)) + 0x44)
#define DEBOUNCE_OFFSET(bk, bkShft)            (((bk) << (bkShft)) + 0x48)
#define EOI_OFFSET(bk, bkShft)                 (((bk) << (bkShft)) + 0x4C)
#define LSSYNC_OFFSET(bk, bkShft)              (((bk) << (bkShft)) + 0x60)
#define IDCODE_OFFSET(bk, bkShft)              (((bk) << (bkShft)) + 0x64)
#define COMPVERSION_OFFSET(bk, bkShft)         (((bk) << (bkShft)) + 0x6C)
#define CONFIGREG1_OFFSET(bk, bkShft)          (((bk) << (bkShft)) + 0x70)
#define CONFIGREG2_OFFSET(bk, bkShft)          (((bk) << (bkShft)) + 0x74)

#define DW_GPIO_INTCFG(i)                      (0x480 + ((i) << 2))

#define GPIO_IRQ_SENSITIVE_NONE             (0x0)
#define GPIO_IRQ_LOW_LEVEL_SENSITIVE        (0x1)
#define GPIO_IRQ_HIGH_LEVEL_SENSITIVE       (0x2)
#define GPIO_IRQ_RISING_EDGE_SENSITIVE      (0x4)
#define GPIO_IRQ_FALLING_EDGE_SENSITIVE     (0x8)
#define GPIO_IRQ_BOTH_EDGE_SENSITIVE        (GPIO_IRQ_RISING_EDGE_SENSITIVE | \
                                             GPIO_IRQ_FALLING_EDGE_SENSITIVE)
#define GPIO_DEFAULT_DEBOUNCE_TIME          (20000u)
#define GPIO_INVALID_DEBOUNCE_TIME          (0xffffffffu)

typedef struct dwGpioDrvCtrl DW_GPIO_DRV_CTRL;

typedef struct dwGpioIntNodeExt
    {
    UINT32             gpioPin;     /* the gpio pin number of this interrupt */
    INTR_TRIGGER       triggerMode; /* record level trigger or edge trigger */
    DW_GPIO_DRV_CTRL * pDrvCtrl;
    } DW_GPIO_INT_NODE_EXT;

typedef struct dwGpioIntNode
    {
    VOIDFUNCPTR             pIsr;               /* ISR */
    void *                  pArg;               /* parameter */
    BOOL                    bEnabled;
    DW_GPIO_INT_NODE_EXT    intExt[0];
    } DW_GPIO_INT_NODE;

/* structure holding the instance specific details */

struct dwGpioDrvCtrl
    {
    VXB_GPIOCTRL        gpioCtrl;       /* must be the first */
    VXB_DEV_ID          gpioDev;
    void *              handle;
#ifdef _WRS_CONFIG_LP64
    UINT64              bar;
#else
    UINT32              bar;
#endif
    void *              intCfgHandle;
    void *              intCfgBar;
    VXB_RESOURCE **     intRes;
    SEM_ID              gpioDevSem;
    UINT32              debounceTime;
    DW_GPIO_INT_NODE *  pIntNode[DW_MAX_GPIO_BANK_WIDTH];
    UINT32              intNum;
    UINT32              portWidth;      /* pin number in each port */
    UINT32              bankNum;        /* bank number the driver supports */
    UINT32              bankShift;
    UINT32              totalPinNum;    /* pin number the driver supports */
    UINT32              portANum;       /* pin number in port A */
    const UINT8 *       pMapTable;      /* IO numbering to GPIO numbering */
};

#define GPIO_BAR(p)       ((p)->bar)
#define GPIO_HANDLE(p)    ((p)->handle)
#define INT_BAR(p)        ((p)->intCfgBar)
#define INT_HANDLE(p)     ((p)->intCfgHandle)

#ifdef __cplusplus
}
#endif

#endif /* __INCvxbFdtSynopsysDwGpioh */
