/* vxbFdtDwXgmacEnd.c - VxBus Intel Agilex5 XGMAC END driver */

/*
 * Copyright (c) 2023 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */


/*

Example .dts entry:

\cs
		xgmac0: ethernet@10810000
            {
			compatible = "snps,dwxgmac";
			reg = <0x10810000 0x3500>;
			interrupts = <222 0 4 0>,
			             <223 0 4 0>,
			             <224 0 4 0>,
			             <225 0 4 0>,
			             <226 0 4 0>,
			             <227 0 4 0>,
			             <228 0 4 0>,
			             <229 0 4 0>,
			             <230 0 4 0>,
			             <231 0 4 0>,
			             <232 0 4 0>,
			             <233 0 4 0>,
			             <234 0 4 0>,
			             <235 0 4 0>,
			             <236 0 4 0>,
			             <237 0 4 0>,
			             <238 0 4 0>;
            phy-mode = "rgmii";
            phy-handle = <&phy0>;
            mac-address = [26 fd bb 85 d8 ce];
            clock-frequency = <250000000>;
            #address-cells = <1>;
            #size-cells = <0>;
            phy0: ethernet-phy@0
                {
                compatible = "marvellPhy";
                reg = <0>;
                };
            };
\ce

*/

#include <vxWorks.h>
#include <intLib.h>
#include <logLib.h>
#include <muxLib.h>
#include <netLib.h>
#include <string.h>
#include <netBufLib.h>
#include <semLib.h>
#include <sysLib.h>
#include <wdLib.h>
#include <etherMultiLib.h>
#include <end.h>
#include <dllLib.h>

#define END_MACROS

#include <endLib.h>
#include <vmLib.h>
#include <hwif/vxBus.h>
#include <hwif/buslib/vxbFdtLib.h>
#include <hwif/methods/vxbMiiMethod.h>
#include <subsys/int/vxbIntLib.h>
#include <subsys/timer/vxbTimerLib.h>
#include <subsys/clk/vxbClkLib.h>
#include <miiBus.h>
#include <vxbEndUtil.h>
#include <vxbIeee1588.h>
#include <private/kwriteLibP.h>

#include "vxbFdtDwXgmacEnd.h"

#define CACHE_LINE_SIZE (64)
#define DESCS_PER_CACHE_LINE (CACHE_LINE_SIZE / sizeof(DMA_DESC))

IMPORT FUNCPTR _func_m2PollStatsIfPoll;

LOCAL STATUS xgmacProbe(VXB_DEV_ID);
LOCAL STATUS xgmacAttach(VXB_DEV_ID);
LOCAL STATUS xgmacMediaAttach(VXB_DEV_ID, VXB_DEV_ID);
LOCAL VOID xgmacHwInit(XGMAC_DRV_CTRL*);
LOCAL VOID xgmacIsr(XGMAC_DRV_CTRL*);
LOCAL VOID xgmacTxRxIntHandle(void*);
LOCAL VOID xgmacTxHandle(XGMAC_DRV_CTRL *, UINT32);
LOCAL VOID xgmacRxHandle(XGMAC_DRV_CTRL *, UINT32);
LOCAL VOID xgmacTxCounterUpdate(XGMAC_DRV_CTRL*, M_BLK_ID);
LOCAL VOID xgmacRxCounterUpdate(XGMAC_DRV_CTRL*, M_BLK_ID);
LOCAL VOID xgmacDmaReset(XGMAC_DRV_CTRL*);
LOCAL VOID xgmacDmaIntEnable(XGMAC_DRV_CTRL*, UINT32);
LOCAL VOID xgmacDmaIntDisable(XGMAC_DRV_CTRL*, UINT32);
LOCAL VOID xgmacRxDmaStart(XGMAC_DRV_CTRL*, UINT32);
LOCAL VOID xgmacTxDmaStart(XGMAC_DRV_CTRL*, UINT32);
LOCAL STATUS xgmacRxDescInit(XGMAC_DRV_CTRL*);
LOCAL STATUS xgmacTxDescInit(XGMAC_DRV_CTRL*);

LOCAL STATUS xgmacMediaUpdate(VXB_DEV_ID);
LOCAL STATUS xgmacMiiRead(VXB_DEV_ID, UINT8, UINT8, UINT16 *);
LOCAL STATUS xgmacMiiWrite(VXB_DEV_ID, UINT8, UINT8, UINT16);
LOCAL VOID   xgmacMdioBusyWait(XGMAC_DRV_CTRL*);

LOCAL VXB_DRV_METHOD vxbFdtDwXgmacMethods [] =
    {
    { VXB_DEVMETHOD_CALL (vxbDevProbe),    (FUNCPTR) xgmacProbe},
    { VXB_DEVMETHOD_CALL (vxbDevAttach),   (FUNCPTR) xgmacAttach},
    { VXB_DEVMETHOD_CALL (miiRead),        (FUNCPTR) xgmacMiiRead},
    { VXB_DEVMETHOD_CALL (miiWrite),       (FUNCPTR) xgmacMiiWrite},
    { VXB_DEVMETHOD_CALL (miiMediaUpdate), (FUNCPTR) xgmacMediaUpdate},
    { VXB_DEVMETHOD_CALL (miiMediaAttach), (FUNCPTR) xgmacMediaAttach},
    VXB_DEVMETHOD_END
    };

VXB_DRV vxbFdtDwXgmacEndDrv =
    {
    {NULL},
    "xgmac",                            /* drvName */
    "XGMAC END driver",                 /* Description */
    VXB_BUSID_FDT,                      /* Class */
    0,                                  /* Flags */
    0,                                  /* Reference count */
    vxbFdtDwXgmacMethods                /* Method table */
    };

#define BUILD_AS_DKM
#undef BUILD_AS_DKM
#ifdef BUILD_AS_DKM
STATUS xgmacDrvAdd(VOID)
    {
    STATUS rc;
    rc = vxbDrvAdd(&vxbFdtDwXgmacEndDrv);
    return rc;
    }
IMPORT int ifconfig(char*);
IMPORT int ipAttach(int,char*);
VOID xgmacConfig(VOID)
    {
    ipAttach(0, "xgmac");
    ifconfig("xgmac0 10.10.0.100 netmask 255.255.255.0 up");
    }
#else
VXB_DRV_DEF (vxbFdtDwXgmacEndDrv)
#endif

/* END functions */

LOCAL END_OBJ*  xgmacLoad(char*, void*);
LOCAL STATUS    xgmacUnload(END_OBJ*);
LOCAL INT32     xgmacIoctl(END_OBJ*, INT32, caddr_t);
LOCAL STATUS    xgmacMCastAddrAdd(END_OBJ*, char*);
LOCAL STATUS    xgmacMCastAddrDel(END_OBJ*, char*);
LOCAL STATUS    xgmacMCastAddrGet(END_OBJ*, MULTI_TABLE*);
LOCAL VOID      xgmacHashTblPopulate(XGMAC_DRV_CTRL*);
LOCAL uint32_t  xgmacBitReverse32(uint32_t);
LOCAL VOID      xgmacRxConfig(XGMAC_DRV_CTRL*);
LOCAL STATUS    xgmacStart(END_OBJ*);
LOCAL STATUS    xgmacStop(END_OBJ*);
LOCAL INT32     xgmacSend(END_OBJ*, M_BLK_ID);
LOCAL INT32     xgmacPollSend(END_OBJ*, M_BLK_ID);
LOCAL INT32     xgmacPollReceive(END_OBJ*, M_BLK_ID);
LOCAL INT32     xgmacEndEncap(XGMAC_DRV_CTRL*, M_BLK_ID, UINT32);
LOCAL UINT32    xgmacTxChnlGet(XGMAC_DRV_CTRL*, M_BLK_ID);

LOCAL NET_FUNCS dwXgmacNetFuncs =
    {
    xgmacStart,
    xgmacStop,
    xgmacUnload,
    xgmacIoctl,
    xgmacSend,
    xgmacMCastAddrAdd,
    xgmacMCastAddrDel,
    xgmacMCastAddrGet,
    xgmacPollSend,
    xgmacPollReceive,
    endEtherAddressForm,
    endEtherPacketDataGet,
    endEtherPacketAddrGet
    };

LOCAL VXB_FDT_DEV_MATCH_ENTRY vxbFdtDwXgmacMatch[] =
    {
        {
        "snps,dwxgmac",  /* compatible */
        NULL
        },
        {}/* Empty terminated list */
    };

XGMAC_DRV_CTRL* gDrvCtrl;

/*******************************************************************************
*
* xgmacProbe - Probe function
*
* RETURNS: OK if the device and driver match up, ERROR otherwise
*
*/

LOCAL STATUS xgmacProbe
    (
    VXB_DEV_ID pDev
    )
    {
    return vxbFdtDevMatch(pDev, vxbFdtDwXgmacMatch, NULL);
    }


/*******************************************************************************
*
* xgmacAttach - Attach function
*
* RETURNS: OK, or ERROR if initialization failed.
*
*/

LOCAL STATUS xgmacAttach
    (
    VXB_DEV_ID pDev
    )
    {
    XGMAC_DRV_CTRL*     pDrvCtrl;
    VXB_FDT_DEV*        pFdtDev;
    VXB_RESOURCE_ADR*   pResAdr = NULL;
    VXB_RESOURCE*       pRes = NULL;
    INT32               proplen = 0;
    const void*         prop;
    VXB_CLK*            pXgmacClk;
    STATUS              rc;
    int                 i;

    VXB_ASSERT (pDev != NULL, ERROR)

    pFdtDev = (VXB_FDT_DEV *) vxbDevIvarsGet(pDev);

    pDrvCtrl = (XGMAC_DRV_CTRL *) vxbMemAlloc(sizeof (XGMAC_DRV_CTRL));
    if (pDrvCtrl == NULL)
        {
        return ERROR;
        }

    gDrvCtrl = pDrvCtrl;
    vxbDevSoftcSet(pDev, pDrvCtrl);
    pDrvCtrl->pDev = pDev;

    pDrvCtrl->unit = vxbDevUnitGet(pDev);

    rc = vxbEndMacAddrGet(pDev, pDrvCtrl->macAddr);
    if (rc != OK)
        {
        goto error;
        }

    pDrvCtrl->mtu = END_MCACHE_CLSIZE;
    prop = vxFdtPropGet(pFdtDev->offset, "jumbo-frame", &proplen);
    if (prop != NULL)
        {
        if (strncmp((char *) prop, "enable", 6) == 0)
            {
            pDrvCtrl->mtu = END_JUMBO_CLSIZE;
            }
        }

    pDrvCtrl->numTuples = XGMAC_NUM_TUPLES;
    pDrvCtrl->hashTableSize = XGMAC_MAX_HASH_TABLE;

    pDrvCtrl->dmaParentTag = vxbDmaBufTagParentGet(pDev, 0);

    pXgmacClk = vxbClkGet(pDev, NULL);
    if (pXgmacClk != NULL)
        {
        pDrvCtrl->clkFreq = (UINT32) vxbClkRateGet(pXgmacClk);
        }

    switch (pDrvCtrl->clkFreq)
        {
        case 250000000:
            pDrvCtrl->mdioCrVal = 1;
            break;
        default:
            pDrvCtrl->mdioCrVal = 0;
        }

    /* Allocate resources */

    pRes = vxbResourceAlloc (pDev, VXB_RES_MEMORY, 0);
    if ((pRes == NULL) || (pRes->pRes == NULL))
        {
        goto error;
        }

    pResAdr = (VXB_RESOURCE_ADR *) pRes->pRes;

    pDrvCtrl->csrBase = (void *) pResAdr->virtual;
    pDrvCtrl->csrHandle = (void *) pResAdr->pHandle;

    pRes = vxbResourceAlloc(pDev, VXB_RES_IRQ, 0);
    if (pRes == NULL)
        {
        goto error;
        }

    pDrvCtrl->macIntRes = pRes;

    for (i = 0; i < XGMAC_NUM_CHNLS; i++)
        {
        pRes = vxbResourceAlloc(pDev, VXB_RES_IRQ, (UINT16)(1 + i));
        if (pRes == NULL)
            {
            goto error;
            }
        pDrvCtrl->txIntRes[i] = pRes;

        pRes = vxbResourceAlloc (pDev, VXB_RES_IRQ, (UINT16)(9 + i));
        if (pRes == NULL)
            {
            goto error;
            }
        pDrvCtrl->rxIntRes[i] = pRes;
        }

    pDrvCtrl->devSem = semMCreate(SEM_Q_PRIORITY | SEM_DELETE_SAFE | SEM_INVERSION_SAFE);
    if (pDrvCtrl->devSem == SEM_ID_NULL)
        {
        goto error;
        }

    /* Create TX and RX descriptor rings */

    pDrvCtrl->txDescRingSize = XGMAC_DESC_RING_SIZE;
    pDrvCtrl->rxDescRingSize = XGMAC_DESC_RING_SIZE;

    pDrvCtrl->txDescDmaTag = vxbDmaBufTagCreate(pDev,
        pDrvCtrl->dmaParentTag, CACHE_LINE_SIZE, 0, VXB_SPACE_MAXADDR, VXB_SPACE_MAXADDR,

        NULL, NULL, sizeof(DMA_DESC) * pDrvCtrl->txDescRingSize, 1,
        sizeof(DMA_DESC) * pDrvCtrl->txDescRingSize,
        VXB_DMABUF_ALLOCNOW|VXB_DMABUF_NOCACHE, NULL, NULL, NULL);
    if (pDrvCtrl->txDescDmaTag == NULL)
        {
        goto error;
        }

    pDrvCtrl->rxDescDmaTag = vxbDmaBufTagCreate(pDev,
        pDrvCtrl->dmaParentTag, CACHE_LINE_SIZE, 0, VXB_SPACE_MAXADDR, VXB_SPACE_MAXADDR,
        NULL, NULL, sizeof(DMA_DESC) * pDrvCtrl->rxDescRingSize, 1,
        sizeof(DMA_DESC) * pDrvCtrl->rxDescRingSize,
        VXB_DMABUF_ALLOCNOW|VXB_DMABUF_NOCACHE, NULL, NULL, NULL);
    if (pDrvCtrl->rxDescDmaTag == NULL)
        {
        goto error;
        }

    for (i = 0; i < XGMAC_NUM_CHNLS; i++)
        {
        DMA_CHNL_INFO* pChInfo;
        pChInfo = &pDrvCtrl->dmaChInfo[i];
        pChInfo->txDescRingCurrent = 0;
        pChInfo->txDescRingTail = 0;
        pChInfo->rxDescRingCurrent = 0;
        pChInfo->rxDescRingTail = 0;
        pChInfo->txDescRingFree = pDrvCtrl->txDescRingSize;

        pChInfo->pTxDesc = vxbDmaBufMemAlloc(pDev, pDrvCtrl->txDescDmaTag, NULL, 0,
            &pChInfo->txDescDmaMap);
        if (pChInfo->pTxDesc == NULL)
            {
            goto error;
            }

        (void) vxbDmaBufMapLoad(pDev, pDrvCtrl->txDescDmaTag, pChInfo->txDescDmaMap,
            pChInfo->pTxDesc, sizeof(DMA_DESC) * pDrvCtrl->txDescRingSize, 0);

        pChInfo->txDescPhysAddr = pChInfo->txDescDmaMap->fragList[0].frag;


        pChInfo->pRxDesc = vxbDmaBufMemAlloc(pDev, pDrvCtrl->rxDescDmaTag, NULL, 0,
            &pChInfo->rxDescDmaMap);
        if (pChInfo->pRxDesc == NULL)
            {
            goto error;
            }

        (void) vxbDmaBufMapLoad(pDev, pDrvCtrl->rxDescDmaTag, pChInfo->rxDescDmaMap,
            pChInfo->pRxDesc, sizeof(DMA_DESC) * pDrvCtrl->rxDescRingSize, 0);

        pChInfo->rxDescPhysAddr = pChInfo->rxDescDmaMap->fragList[0].frag;
        }

    /* Create tag for mBlk mappings */

    pDrvCtrl->mBlkDmaTag = vxbDmaBufTagCreate(pDev, pDrvCtrl->dmaParentTag,
        1, 0, VXB_SPACE_MAXADDR, VXB_SPACE_MAXADDR, NULL, NULL,
        (bus_size_t) pDrvCtrl->mtu, XGMAC_MAXFRAG, (bus_size_t) pDrvCtrl->mtu,
        VXB_DMABUF_ALLOCNOW, NULL, NULL, NULL);
    if (pDrvCtrl->mBlkDmaTag == NULL)
        {
        goto error;
        }

    /* Connect interrupts */

    (void) vxbIntConnect(pDev, pDrvCtrl->macIntRes, xgmacIsr, pDrvCtrl);

    for (i = 0; i < XGMAC_NUM_CHNLS; i++)
        {
        (void) vxbIntConnect(pDev, pDrvCtrl->txIntRes[i], xgmacIsr, pDrvCtrl);
        (void) vxbIntConnect(pDev, pDrvCtrl->rxIntRes[i], xgmacIsr, pDrvCtrl);
        }

    /* Create MII bus */

    rc = miiBusCreate(pDev, VXB_MDIO_FLAG_CLAUSE22);
    if (rc != OK)
        {
        goto error;
        }

    if (pDrvCtrl->pMiiDev == NULL)
        {
        goto error;
        }

    rc = miiBusMediaListGet(pDrvCtrl->pMiiDev, &pDrvCtrl->pMediaList);
    if (rc != OK)
        {
        goto error;
        }

    rc = miiBusModeSet(pDrvCtrl->pMiiDev,
        pDrvCtrl->pMediaList->endMediaListDefault);
    if (rc != OK)
        {
        goto error;
        }

    /* Load and start the END */

    pDrvCtrl->cookie = muxDevLoad((int) pDrvCtrl->unit, xgmacLoad,
        "", TRUE, pDev);
    if (pDrvCtrl->cookie == NULL)
        {
        goto error;
        }

    (void) muxDevStart(pDrvCtrl->cookie);

    if (_func_m2PollStatsIfPoll != NULL)
        {
        (void) endPollStatsInit(pDrvCtrl->cookie, _func_m2PollStatsIfPoll);
        }
    return OK;

error:
    for (i = 0; i < XGMAC_NUM_CHNLS; i++)
        {
        DMA_CHNL_INFO* pChInfo;
        pChInfo = &pDrvCtrl->dmaChInfo[i];

        if (pChInfo->pRxDesc != NULL)
            {
            (void) vxbDmaBufMapUnload(pDrvCtrl->rxDescDmaTag, pChInfo->rxDescDmaMap);
            (void) vxbDmaBufMemFree(pDrvCtrl->rxDescDmaTag, pChInfo->pRxDesc, pChInfo->rxDescDmaMap);
            }
        if (pChInfo->pTxDesc != NULL)
            {
            (void) vxbDmaBufMapUnload(pDrvCtrl->txDescDmaTag, pChInfo->txDescDmaMap);
            (void) vxbDmaBufMemFree(pDrvCtrl->txDescDmaTag, pChInfo->pTxDesc, pChInfo->txDescDmaMap);
            }

        }

    if (pDrvCtrl->devSem != NULL)
        {
        (void) semDelete(pDrvCtrl->devSem);
        }

    if (pDrvCtrl != NULL)
        {
        vxbMemFree(pDrvCtrl);
        }

    return ERROR;
    }


/*******************************************************************************
*
* xgmacTxDescInit - Initialize TX descriptors
*
*/

LOCAL STATUS xgmacTxDescInit
    (
    XGMAC_DRV_CTRL* pDrvCtrl
    )
    {
    DMA_CHNL_INFO* pChInfo;
    DMA_DESC_INFO* pDescInfo;
    UINT32 i;
    UINT32 j;

    for (i = 0; i < XGMAC_NUM_CHNLS; i++)
        {
        pChInfo = &pDrvCtrl->dmaChInfo[i];
        for (j = 0; j < pDrvCtrl->txDescRingSize; j++)
            {
            pDescInfo = &pChInfo->txDescInfo[j];
            (void) vxbDmaBufMapCreate(pDrvCtrl->pDev, pDrvCtrl->mBlkDmaTag, 0, &pDescInfo->map);
            if (pDescInfo->map == NULL)
                {
                return ERROR;
                }
            }
        }

    return OK;
    }

/*******************************************************************************
*
* xgmacRxDescInit - Initialize RX descriptors
*
*/

LOCAL STATUS xgmacRxDescInit
    (
    XGMAC_DRV_CTRL* pDrvCtrl
    )
    {
    M_BLK_ID pMblk;
    DMA_CHNL_INFO* pChInfo;
    DMA_DESC* pDesc;
    DMA_DESC_INFO* pDescInfo;
    PHYS_ADDR physAddr;
    VXB_DMA_MAP_ID pMap;
    STATUS rc;
    UINT32 i;
    UINT32 j;

    for (i = 0; i < XGMAC_NUM_CHNLS; i++)
        {
        pChInfo = &pDrvCtrl->dmaChInfo[i];
        for (j = 0; j < pDrvCtrl->rxDescRingSize; j++)
            {
            pDesc = &pChInfo->pRxDesc[j];
            pDescInfo = &pChInfo->rxDescInfo[j];
            (void) vxbDmaBufMapCreate(pDrvCtrl->pDev, pDrvCtrl->mBlkDmaTag, 0, &pDescInfo->map);
            if (pDescInfo->map == NULL)
                {
                return ERROR;
                }

            pMap = pDescInfo->map;

            pMblk = endPoolTupleGet(pDrvCtrl->endObj.pNetPool);
            if (pMblk == NULL)
                {
                return ERROR;
                }
            pDescInfo->pMblk = pMblk;

            rc = vxbDmaBufMapLoad(pDrvCtrl->pDev, pDrvCtrl->mBlkDmaTag, pMap,
                pMblk->m_data, (bus_size_t)(pDrvCtrl->mtu), 0);
            if (rc != OK)
                {
                return ERROR;
                }

            rc = vxbDmaBufSync(pDrvCtrl->pDev, pDrvCtrl->mBlkDmaTag, pMap,
                _VXB_DMABUFSYNC_DMA_PREREAD);
            if (rc != OK)
                {
                return ERROR;
                }

            physAddr = pMap->fragList[0].frag;
            pDesc->desc0 = VXB_ADDR_LOW32(physAddr);
            pDesc->desc1 = VXB_ADDR_HIGH32(physAddr);
            pDesc->desc2 = 0;
            pDesc->desc3 = XGMAC_RDES3_OWN | XGMAC_RDES3_IOC;
            }

        pChInfo->rxDescRingTail = pDrvCtrl->rxDescRingSize - 1;
        }

    return OK;
    }


/*******************************************************************************
*
* xgmacHwInit - Initialize registers
*
*/

LOCAL VOID xgmacHwInit
    (
    XGMAC_DRV_CTRL* pDrvCtrl
    )
    {
    UINT32 val;
    UINT32 i;

    xgmacDmaReset(pDrvCtrl);

    /* Match linux value for DMA_SYS_BUS_MODE and XGMAC_PACKET_FILTER */

    CSR_WRITE_4(pDrvCtrl, DMA_SYS_BUS_MODE, 0x1f1f081f);
    CSR_WRITE_4(pDrvCtrl, XGMAC_PACKET_FILTER, 0x10404);

    /* Clear MAC addresses */

    for (i = 0; i < 32; i++)
        {
        CSR_WRITE_4(pDrvCtrl, XGMAC_ADDRx_LOW(i), 0);
        CSR_WRITE_4(pDrvCtrl, XGMAC_ADDRx_HIGH(i), 0);
        }

    /* Set MAC address */

    val = pDrvCtrl->macAddr[0] | (UINT32) pDrvCtrl->macAddr[1] << 8 |
        (UINT32) pDrvCtrl->macAddr[2] << 16 | (UINT32) pDrvCtrl->macAddr[3] << 24;
    CSR_WRITE_4(pDrvCtrl, XGMAC_ADDRx_LOW(0), val);
    val = pDrvCtrl->macAddr[4] | (UINT32) pDrvCtrl->macAddr[5] << 8;
    CSR_WRITE_4(pDrvCtrl, XGMAC_ADDRx_HIGH(0), val);

    for (i = 0; i < XGMAC_NUM_CHNLS; i++)
        {
        DMA_CHNL_INFO* pChInfo;
        pChInfo = &pDrvCtrl->dmaChInfo[i];

        /* RX descriptor registers */

        CSR_WRITE_4(pDrvCtrl, XGMAC_DMA_CH_RxDESC_LADDR(i), VXB_ADDR_LOW32(pChInfo->rxDescPhysAddr));
        CSR_WRITE_4(pDrvCtrl, XGMAC_DMA_CH_RxDESC_HADDR(i), VXB_ADDR_HIGH32(pChInfo->rxDescPhysAddr));
        CSR_WRITE_4(pDrvCtrl, XGMAC_DMA_CH_RxDESC_RING_LEN(i), pDrvCtrl->rxDescRingSize - 1);
        CSR_WRITE_4(pDrvCtrl, XGMAC_DMA_CH_RxDESC_TAIL_LPTR(i),
            (VXB_ADDR_LOW32(pChInfo->rxDescPhysAddr) + pChInfo->rxDescRingTail * sizeof(DMA_DESC)));

        /* TX descriptor registers */

        CSR_WRITE_4(pDrvCtrl, XGMAC_DMA_CH_TxDESC_LADDR(i), VXB_ADDR_LOW32(pChInfo->txDescPhysAddr));
        CSR_WRITE_4(pDrvCtrl, XGMAC_DMA_CH_TxDESC_HADDR(i), VXB_ADDR_HIGH32(pChInfo->txDescPhysAddr));
        CSR_WRITE_4(pDrvCtrl, XGMAC_DMA_CH_TxDESC_RING_LEN(i), pDrvCtrl->txDescRingSize - 1);
        CSR_WRITE_4(pDrvCtrl, XGMAC_DMA_CH_TxDESC_TAIL_LPTR(i),
            (VXB_ADDR_LOW32(pChInfo->txDescPhysAddr) + pChInfo->txDescRingTail * sizeof(DMA_DESC)));
        }

    /* Initialize TX */

    val = XGMAC_CORE_INIT_TX;
    CSR_WRITE_4(pDrvCtrl, XGMAC_TX_CONFIG, val);

    /* Initialize RX */

    val = XGMAC_CORE_INIT_RX | XGMAC_CONFIG_CST;
    CSR_WRITE_4(pDrvCtrl, XGMAC_RX_CONFIG, val);

    /* Enable MAC interrupts */

    CSR_WRITE_4(pDrvCtrl, XGMAC_INT_EN, XGMAC_INT_DEFAULT_EN);

    /* Set operating mode for each TX and RX queue (use Linux values) */

    for (i = 0; i < XGMAC_NUM_CHNLS; i++)
        {
        CSR_WRITE_4(pDrvCtrl, XGMAC_MTL_TXQ_OPMODE(i), 0xf000a);
        CSR_WRITE_4(pDrvCtrl, XGMAC_MTL_RXQ_OPMODE(i), 0x70020);
        }

    /* Enable RX queues 0 - 7 */

    CSR_WRITE_4(pDrvCtrl, XGMAC_RXQ_CTRL0, 0xaaaa);

    /* Enable channel interrupts and start each channel */

    for (i = 0; i < XGMAC_NUM_CHNLS; i++)
        {
        xgmacDmaIntEnable(pDrvCtrl, i);
        xgmacRxDmaStart(pDrvCtrl, i);
        xgmacTxDmaStart(pDrvCtrl, i);
        }
    }


/*******************************************************************************
*
* xgmacDmaReset - Reset hardware
*
*/

LOCAL VOID xgmacDmaReset
    (
    XGMAC_DRV_CTRL* pDrvCtrl
    )
    {
    UINT32 val;

    val = CSR_READ_4(pDrvCtrl, XGMAC_DMA_MODE);
    val |= XGMAC_SWR;
    CSR_WRITE_4(pDrvCtrl, XGMAC_DMA_MODE, val);
    for (;;)
        {
        val = CSR_READ_4(pDrvCtrl, XGMAC_DMA_MODE);
        if ((val & XGMAC_SWR) == 0)
            {
            break;
            }
        }
    }


/*******************************************************************************
*
* xgmacDmaIntEnable - Enable channel interrupts
*
*/

LOCAL VOID xgmacDmaIntEnable
    (
    XGMAC_DRV_CTRL* pDrvCtrl,
    UINT32 ch
    )
    {
    CSR_SETBIT_4(pDrvCtrl, XGMAC_DMA_CH_INT_EN(ch), XGMAC_DMA_INT_DEFAULT_EN);
    }


/*******************************************************************************
*
* xgmacDmaIntDisable - Disable channel interrupts
*
*/

LOCAL VOID xgmacDmaIntDisable
    (
    XGMAC_DRV_CTRL* pDrvCtrl,
    UINT32 ch
    )
    {
    CSR_CLRBIT_4(pDrvCtrl, XGMAC_DMA_CH_INT_EN(ch), XGMAC_DMA_INT_DEFAULT_EN);
    }


/*******************************************************************************
*
* xgmacRxDmaStart - Start RX DMA
*
*/

LOCAL VOID xgmacRxDmaStart
    (
    XGMAC_DRV_CTRL* pDrvCtrl,
    UINT32 ch
    )
    {
    UINT32 val;

    val = CSR_READ_4(pDrvCtrl, XGMAC_DMA_CH_RX_CONTROL(ch));
    val &= ~XGMAC_RBSZ;
    val |= pDrvCtrl->mtu << XGMAC_RBSZ_SHIFT;
    val |= XGMAC_RXST;
    CSR_WRITE_4(pDrvCtrl, XGMAC_DMA_CH_RX_CONTROL(ch), val);
    CSR_SETBIT_4(pDrvCtrl, XGMAC_RX_CONFIG, XGMAC_CONFIG_RE);
    }


/*******************************************************************************
*
* xgmacTxDmaStart - Start TX DMA
*
*/

LOCAL VOID xgmacTxDmaStart
    (
    XGMAC_DRV_CTRL* pDrvCtrl,
    UINT32 ch
    )
    {
    UINT32 txpbl = 0x20;
    UINT32 val;

    val = XGMAC_PBLx8 | 1; // Set MSS to 1
    CSR_WRITE_4(pDrvCtrl, XGMAC_DMA_CH_CONTROL(ch), val);

    val = CSR_READ_4(pDrvCtrl, XGMAC_DMA_CH_TX_CONTROL(ch));
    val &= ~XGMAC_TxPBL;
    val |= (txpbl << XGMAC_TxPBL_SHIFT) & XGMAC_TxPBL;
    val |= XGMAC_TSE | XGMAC_TXST;
    CSR_WRITE_4(pDrvCtrl, XGMAC_DMA_CH_TX_CONTROL(ch), val);
    }


/*******************************************************************************
*
* xgmacLoad - END load function
*
* RETURNS: An END object pointer, or NULL on error, or 0 and the name of the
* device if the <loadStr> was empty.
*
*/

LOCAL END_OBJ *xgmacLoad
    (
    char* loadStr,
    void* pArg
    )
    {
    VXB_DEV_ID pDev;
    XGMAC_DRV_CTRL* pDrvCtrl;
    STATUS rc;

    if (loadStr == NULL)
        {
        return NULL;
        }

    if (loadStr[0] == 0)
        {
        memcpy(loadStr, XGMAC_NAME, sizeof(XGMAC_NAME));
        return NULL;
        }

    pDev = pArg;
    pDrvCtrl = vxbDevSoftcGet(pDev);

    rc = END_OBJ_INIT(&pDrvCtrl->endObj, NULL, XGMAC_NAME,
        pDrvCtrl->unit, &dwXgmacNetFuncs, "XGMAC END driver");
    if (rc == ERROR)
        {
        return NULL;
        }

    rc = endM2Init (&pDrvCtrl->endObj, M2_ifType_ethernet_csmacd,
        pDrvCtrl->macAddr, ETHER_ADDR_LEN, ETHERMTU, 1000000000,
        IFF_NOTRAILERS | IFF_SIMPLEX | IFF_MULTICAST | IFF_BROADCAST);
    if (rc == ERROR)
        {
        return NULL;
        }

    if (pDrvCtrl->mtu == END_MCACHE_CLSIZE)
        {
        rc = endPoolCreate((int) pDrvCtrl->numTuples, &pDrvCtrl->endObj.pNetPool);
        }
    else
        {
        rc = endPoolJumboCreate((int) pDrvCtrl->numTuples, &pDrvCtrl->endObj.pNetPool);
        }

    if (rc == ERROR)
        {
        return NULL;
        }

    pDrvCtrl->endStatsConf.ifPollInterval = (int)vxbSysClkRateGet ();
    pDrvCtrl->endStatsConf.ifEndObj = &pDrvCtrl->endObj;
    pDrvCtrl->endStatsConf.ifWatchdog = NULL;
    pDrvCtrl->endStatsConf.ifValidCounters = (
        END_IFINUCASTPKTS_VALID |
        END_IFINMULTICASTPKTS_VALID |
        END_IFINBROADCASTPKTS_VALID |
        END_IFINOCTETS_VALID |
        END_IFINERRORS_VALID |
        END_IFINDISCARDS_VALID |
        END_IFOUTUCASTPKTS_VALID |
        END_IFOUTMULTICASTPKTS_VALID |
        END_IFOUTBROADCASTPKTS_VALID |
        END_IFOUTOCTETS_VALID |
        END_IFOUTERRORS_VALID);

    pDrvCtrl->endCaps.cap_available = IFCAP_VLAN_MTU;
    pDrvCtrl->endCaps.cap_enabled = IFCAP_VLAN_MTU;

    if (pDrvCtrl->mtu != END_MCACHE_CLSIZE)
        {
        pDrvCtrl->endCaps.cap_available |= IFCAP_JUMBO_MTU;
        pDrvCtrl->endCaps.cap_enabled |= IFCAP_JUMBO_MTU;
        }

    return &pDrvCtrl->endObj;
    }


/*******************************************************************************
*
* xgmacUnload - unload END driver instance
*
* RETURNS: ERROR if device is still in the IFF_UP state, otherwise OK
*
*/

LOCAL STATUS xgmacUnload
    (
    END_OBJ * pEnd    /*pointer to end object to be distroyed */
    )
    {
    XGMAC_DRV_CTRL * pDrvCtrl;

    /* we must be stopped before we can be unloaded */

    if ((pEnd->flags & IFF_UP) != 0)
        {
        return (ERROR);
        }

    pDrvCtrl = (XGMAC_DRV_CTRL *) pEnd;

    /* relase our buffer pool */

    (void)endPoolDestroy (pDrvCtrl->endObj.pNetPool);

    /* terminate stats polling */

    (void)wdDelete (pDrvCtrl->endStatsConf.ifWatchdog);

    (void)endM2Free (&pDrvCtrl->endObj);

    END_OBJECT_UNLOAD (&pDrvCtrl->endObj);

    /* prevent freeing of pDrvCtrl */

    return (EALREADY);
    }


/*******************************************************************************
*
* xgmacMCastAddrAdd - add a multicast address
*
* RETURNS: OK or ERROR
*
* ERRNO: N/A
*/

LOCAL STATUS xgmacMCastAddrAdd
    (
    END_OBJ* pEnd,
    char* pAddr
    )
    {
    XGMAC_DRV_CTRL * pDrvCtrl;
    INT32  ret;
    STATUS rc;

    pDrvCtrl = (XGMAC_DRV_CTRL *) pEnd;

    (void) semTake(pDrvCtrl->devSem, WAIT_FOREVER);

    ret = etherMultiAdd(&pEnd->multiList, pAddr);
    if (ret == ENETRESET)
        {
        pEnd->nMulti++;
        xgmacHashTblPopulate((XGMAC_DRV_CTRL *) pEnd);
        rc = OK;
        }
    else if (ret == OK)
        {
        rc = OK;
        }
    else
        {
        rc = ERROR;
        }

    (void) semGive(pDrvCtrl->devSem);

    return rc;
    }

/*******************************************************************************
*
* xgmacMCastAddrDel - delete a multicast address
*
* RETURNS: OK or ERROR
*
*/

LOCAL STATUS xgmacMCastAddrDel
    (
    END_OBJ* pEnd,
    char*    pAddr
    )
    {
    XGMAC_DRV_CTRL* pDrvCtrl;
    INT32 ret;
    STATUS rc;

    pDrvCtrl = (XGMAC_DRV_CTRL *) pEnd;

    (void) semTake(pDrvCtrl->devSem, WAIT_FOREVER);
    ret = etherMultiDel(&pEnd->multiList, pAddr);

    if (ret == ENETRESET)
        {
        pEnd->nMulti--;
        xgmacHashTblPopulate((XGMAC_DRV_CTRL *) pEnd);
        rc = OK;
        }
    else if (ret == OK)
        {
        rc = OK;
        }
    else
        {
        rc = ERROR;
        }

    (void) semGive(pDrvCtrl->devSem);

    return rc;
    }


/*******************************************************************************
*
* xgmacHashTblPopulate - populate the multicast hash filter
*
*/

LOCAL VOID xgmacHashTblPopulate
    (
    XGMAC_DRV_CTRL* pDrvCtrl
    )
    {
    ETHER_MULTI *   mCastNode = NULL;
    uint32_t        mcFilter [8] ={0};
    uint32_t        bitNr;
    UINT32          filter;
    uint32_t        ix = 0;
    int             shift;

    if (pDrvCtrl->hashTableSize == 0)
        {
        return;
        }

    filter = CSR_READ_4(pDrvCtrl, XGMAC_PACKET_FILTER);
    filter &= ~(XGMAC_FILTER_PR | XGMAC_FILTER_HMC | XGMAC_FILTER_PM);
    filter |= XGMAC_FILTER_HPF;

    if (pDrvCtrl->endObj.flags & IFF_ALLMULTI)
        {
        /* set all multicast mode */

        for (ix = 0; ix < pDrvCtrl->hashTableSize; ix++)
            {
            CSR_WRITE_4 (pDrvCtrl, XGMAC_HASH_TABLE(ix), 0xffffffffU);
            vxbUsDelay (1);
            }

        filter |= XGMAC_FILTER_PM;
        CSR_WRITE_4(pDrvCtrl, XGMAC_PACKET_FILTER, filter);
        return;
        }

    for (ix = 0; ix < pDrvCtrl->hashTableSize; ix++)
        {
        CSR_WRITE_4 (pDrvCtrl, XGMAC_HASH_TABLE(ix), 0);
        vxbUsDelay (1);
        }

    shift = pDrvCtrl->hashTableSize == 2 ? 26 :
            pDrvCtrl->hashTableSize == 4 ? 25 : 24;

    for (mCastNode = (ETHER_MULTI *) lstFirst (&pDrvCtrl->endObj.multiList);
         mCastNode != NULL;
         mCastNode = (ETHER_MULTI *) lstNext (&mCastNode->node))
        {
        /*
         * The upper 8 bits of the calculated CRC are used to
         * index the contents of the hash table.
         */

        bitNr = endEtherCrc32LeGet((const uint8_t *) mCastNode->addr, ETHER_ADDR_LEN);
        bitNr = xgmacBitReverse32(~bitNr) >> shift;

        mcFilter[bitNr >> 5] |= (uint32_t)(1 << (bitNr & 31));

        XGMAC_DBG(DBG_INFO, "Add multicast address [%02X:%02X:%02X:%02X:%02X:%02X]\n",
            (uint8_t)mCastNode->addr[0], (uint8_t)mCastNode->addr[1],
            (uint8_t)mCastNode->addr[2], (uint8_t)mCastNode->addr[3],
            (uint8_t)mCastNode->addr[4], (uint8_t)mCastNode->addr[5]);
        }

    for (ix = 0; ix < pDrvCtrl->hashTableSize; ix++)
        {
        CSR_WRITE_4 (pDrvCtrl, XGMAC_HASH_TABLE(ix), mcFilter[ix]);
        vxbUsDelay (1);
        }

    filter |= XGMAC_FILTER_HMC;
    CSR_WRITE_4(pDrvCtrl, XGMAC_PACKET_FILTER, filter);
    }


/*******************************************************************************
*
* xgmacBitReverse32 - get 32bit bit reverse value.
*
* This function compute 32bit bit reverse value.
*
* RETURNS: bit reversed value.
*
*/

LOCAL uint32_t xgmacBitReverse32
    (
    uint32_t input
    )
    {
    uint32_t ix = 0;
    uint32_t output = 0;

    for (; ix < 32; ix++)
        {
        output |= (uint32_t)(((input & (uint32_t)(1 << ix)) != 0) << (31 - ix));
        }

    return output;
    }


/*******************************************************************************
 *
 * xgmacMCastAddrGet - get the multicast address list for the device
 *
 * This routine gets the multicast list of whatever the driver is already
 * listening for.
 *
 * RETURNS: OK or ERROR.
 *
 * ERRNO: N/A
 */

LOCAL STATUS xgmacMCastAddrGet
    (
    END_OBJ *     pEnd,   /* pointer to the end object */
    MULTI_TABLE * pTable  /* pointer to table */
    )
    {
    XGMAC_DRV_CTRL *pDrvCtrl;
    STATUS         state;
    pDrvCtrl = (XGMAC_DRV_CTRL *) pEnd;

    (void)semTake (pDrvCtrl->devSem, WAIT_FOREVER);

    state = etherMultiGet (&pEnd->multiList, pTable);

    (void)semGive (pDrvCtrl->devSem);

    return state;
    }


/*******************************************************************************
*
* xgmacIoctl - the driver I/O control routine
*
* This function processes ioctl requests supplied via the muxIoctl() routine.
* In addition to the normal boilerplate END ioctls, this driver supports the
* IFMEDIA ioctls, END capabilities ioctls, and polled stats ioctls.
*
* RETURNS: A command specific response, usually OK or ERROR.
*
* ERRNO: N/A
*/

LOCAL INT32 xgmacIoctl
    (
    END_OBJ * pEnd,   /* pointer to end object */
    INT32     cmd,    /* command */
    caddr_t   data    /* data */
    )
    {
    XGMAC_DRV_CTRL *    pDrvCtrl;
    END_MEDIALIST *    mediaList;
    END_CAPABILITIES * hwCaps;
    END_MEDIA *        pMedia;
    END_RCVJOBQ_INFO * qinfo;
    UINT32             nQs;
    INT32              value;
    INT32              error = OK;
    STATUS             rc;

    pDrvCtrl = (XGMAC_DRV_CTRL *) pEnd;

    if (cmd != EIOCPOLLSTART && cmd != EIOCPOLLSTOP)
        {
        (void) semTake(pDrvCtrl->devSem, WAIT_FOREVER);
        }

    switch (cmd)
        {
        case EIOCSADDR:

            if (data == NULL)
                {
                error = EINVAL;
                }
            else
                {
                bcopy ((char *) data, (char *) pDrvCtrl->macAddr,
                       ETHER_ADDR_LEN);
                bcopy ((char *) data,
                       (char *) pEnd->mib2Tbl.ifPhysAddress.phyAddress,
                       ETHER_ADDR_LEN);
                if (pEnd->pMib2Tbl != NULL)
                    bcopy ((char *) data,
                           (char *) pEnd->pMib2Tbl->m2Data.mibIfTbl.ifPhysAddress.phyAddress,
                           ETHER_ADDR_LEN);

                xgmacRxConfig(pDrvCtrl);
                }
            break;

        case EIOCGADDR:

            if (data == NULL)
                error = EINVAL;
            else
                bcopy ((char *) pDrvCtrl->macAddr, (char *) data,
                       ETHER_ADDR_LEN);
            break;

        case EIOCSFLAGS:

#ifndef _WRS_CONFIG_LP64
            value = (INT32) data;
#else /* _WRS_CONFIG_LP64 */
            value = (INT32)((INT64) data);
#endif /* !_WRS_CONFIG_LP64 */

            if (value < 0)
                {
                value = -value;
                value--;
                END_FLAGS_CLR (pEnd, value);
                }
            else
                END_FLAGS_SET (pEnd, value);

            xgmacRxConfig(pDrvCtrl);
            break;

        case EIOCGFLAGS:

            if (data == NULL)
                error = EINVAL;
            else
                *(INT32 *) data = END_FLAGS_GET (pEnd);
            break;

        case EIOCMULTIADD:

            error = xgmacMCastAddrAdd (pEnd, (char *) data);
            break;

        case EIOCMULTIDEL:

            error = xgmacMCastAddrDel (pEnd, (char *) data);
            break;

        case EIOCMULTIGET:

            error = xgmacMCastAddrGet (pEnd, (MULTI_TABLE *) data);
            break;

        case EIOCPOLLSTART:
            pDrvCtrl->polling = TRUE;
            break;

        case EIOCPOLLSTOP:
            pDrvCtrl->polling = FALSE;
            break;

        case EIOCGMIB2233:
        case EIOCGMIB2:

            error = endM2Ioctl(&pDrvCtrl->endObj, cmd, data);
            break;

        case EIOCGPOLLCONF:

            if (data == NULL)
                error = EINVAL;
            else
                {
                pDrvCtrl->endStatsConf.ifPollInterval = (int)sysClkRateGet();
                *((END_IFDRVCONF **) data) = &pDrvCtrl->endStatsConf;
                }
            break;

        case EIOCGPOLLSTATS:
            XGMAC_DBG (DBG_IOCRL, "EIOCGPOLLSTATS\n");

            if (data == NULL)
                {
                error = EINVAL;
                }
            else
                {
                pDrvCtrl->endStatsCountersShared = pDrvCtrl->endStatsCounters;
                bzero (&pDrvCtrl->endStatsCounters, sizeof(END_IFCOUNTERS));
                *((END_IFCOUNTERS **)data) = &pDrvCtrl->endStatsCountersShared;
                }
            break;

        case EIOCGMEDIALIST:

            if (data == NULL)
                {
                error = EINVAL;
                break;
                }
            if (pDrvCtrl->pMediaList->endMediaListLen == 0)
                {
                error = ENOTSUP;
                break;
                }

            mediaList = (END_MEDIALIST *) data;
            if (mediaList->endMediaListLen
                    < pDrvCtrl->pMediaList->endMediaListLen)
                {
                mediaList->endMediaListLen = pDrvCtrl->pMediaList->endMediaListLen;
                error = ENOSPC;
                break;
                }

            bcopy ((char *) pDrvCtrl->pMediaList, (char *) mediaList,
                    sizeof (END_MEDIALIST) +
                    sizeof (UINT32) * pDrvCtrl->pMediaList->endMediaListLen);
            break;

        case EIOCGIFMEDIA:

            if (data == NULL)
                error = EINVAL;
            else
                {
                pMedia = (END_MEDIA *) data;
                pMedia->endMediaActive = pDrvCtrl->miiMode;
                pMedia->endMediaStatus = pDrvCtrl->miiStatus;
                }
            break;

        case EIOCSIFMEDIA:

            if (data == NULL)
                {
                error = EINVAL;
                }
            else
                {
                pMedia = (END_MEDIA *) data;
                (void)miiBusModeSet(pDrvCtrl->pMiiDev, pMedia->endMediaActive);
                rc = xgmacMediaUpdate(pDrvCtrl->pDev);
                if (rc != OK)
                    {
                    error = ERROR;
                    }
                else
                    error = OK;
                }
            break;

        case EIOCGIFCAP:

            hwCaps = (END_CAPABILITIES *) data;
            if (hwCaps == NULL)
                {
                error = EINVAL;
                break;
                }
            hwCaps->cap_available = pDrvCtrl->endCaps.cap_available;
            hwCaps->cap_enabled = pDrvCtrl->endCaps.cap_enabled;
            break;

            /*
             * The only special capability we support is VLAN_MTU, and
             * it can never be turned off.
             */

        case EIOCSIFCAP:

            error = ENOTSUP;
            break;

        case EIOCGIFMTU:

            if (data == NULL)
                error = EINVAL;
            else
                *(INT32 *) data = pEnd->mib2Tbl.ifMtu;
            break;

        case EIOCSIFMTU:

#ifndef _WRS_CONFIG_LP64
            value = (INT32) data;
#else  /* _WRS_CONFIG_LP64 */
            value = (INT32)((INT64) data);
#endif /* !_WRS_CONFIG_LP64 */

            if (value <= 0 || value > (INT32) pDrvCtrl->mtu)
                {
                error = EINVAL;
                break;
                }
            pEnd->mib2Tbl.ifMtu = value;
            if (pEnd->pMib2Tbl != NULL)
                pEnd->pMib2Tbl->m2Data.mibIfTbl.ifMtu = value;
            break;

        case EIOCGRCVJOBQ:
            XGMAC_DBG (DBG_IOCRL, "EIOCGRCVJOBQ\n");

            if (data == NULL)
                {
                error = EINVAL;
                break;
                }

            qinfo = (END_RCVJOBQ_INFO *)data;
            nQs = qinfo->numRcvJobQs;
            qinfo->numRcvJobQs = 1;

            if (nQs < 1)
                {
                error = ENOSPC;
                }
            else
                {
                qinfo->qIds[0] = pDrvCtrl->jobQueue;
                }

            break;

        default:
            error = EINVAL;
            break;
        }

    if (cmd != EIOCPOLLSTART && cmd != EIOCPOLLSTOP)
        {
        (void) semGive(pDrvCtrl->devSem);
        }

    return (error);
    }


/*******************************************************************************
*
* xgmacRxConfig - Set MAC address and RX filter
*
*/

LOCAL VOID xgmacRxConfig
    (
    XGMAC_DRV_CTRL* pDrvCtrl
    )
    {
    UINT32 val;

    /* Set MAC address */

    val = pDrvCtrl->macAddr[0] | (UINT32) pDrvCtrl->macAddr[1] << 8 |
        (UINT32) pDrvCtrl->macAddr[2] << 16 | (UINT32) pDrvCtrl->macAddr[3] << 24;
    CSR_WRITE_4(pDrvCtrl, XGMAC_ADDRx_LOW(0), val);
    val = pDrvCtrl->macAddr[4] | (UINT32) pDrvCtrl->macAddr[5] << 8;
    CSR_WRITE_4(pDrvCtrl, XGMAC_ADDRx_HIGH(0), val);

    /* Promisc. mode specified? */

    if (pDrvCtrl->endObj.flags & IFF_PROMISC)
        {
        CSR_SETBIT_4(pDrvCtrl, XGMAC_PACKET_FILTER, XGMAC_FILTER_PR);
        }
    else
        {
        CSR_CLRBIT_4(pDrvCtrl, XGMAC_PACKET_FILTER, XGMAC_FILTER_PR);
        }

    xgmacHashTblPopulate(pDrvCtrl);
    }


/*******************************************************************************
*
* xgmacStart - Start the device
*
* This function resets the device to put it into a known state and then
* configures it for RX and TX operation. The RX and TX configuration registers
* are initialized, and the address of the RX and TX DMA rings are loaded into
* the device. Interrupts are then enabled, and the initial link state is
* configured.
*
* RETURNS: ERROR if device initialization failed, otherwise OK
*
* ERRNO: N/A
*/

LOCAL STATUS xgmacStart
    (
    END_OBJ* pEnd
    )
    {
    XGMAC_DRV_CTRL * pDrvCtrl;
    STATUS rc;
    int i;

    pDrvCtrl = (XGMAC_DRV_CTRL *) pEnd;

    rc = semTake(pDrvCtrl->devSem, WAIT_FOREVER);
    if (rc != OK)
        {
        return ERROR;
        }

    rc = END_TX_SEM_TAKE(pEnd, WAIT_FOREVER);
    if (rc != OK)
        {
        (void) semGive(pDrvCtrl->devSem);
        return ERROR;
        }

    for (i = 0; i < XGMAC_NUM_CHNLS; i++)
        {
        DMA_CHNL_INFO* pChInfo;
        pChInfo = &pDrvCtrl->dmaChInfo[i];
        vxAtomicSet(&pChInfo->intStatus, 0);
        vxAtomicSet(&pChInfo->intPending, 0);
        }

    rc = xgmacTxDescInit(pDrvCtrl);
    if (rc != OK)
        {
        goto exit;
        }

    rc = xgmacRxDescInit(pDrvCtrl);
    if (rc != OK)
        {
        goto exit;
        }

    pDrvCtrl->miiMode = IFM_ETHER | IFM_NONE;
    pDrvCtrl->miiStatus = IFM_AVALID;

    rc = miiBusModeSet(pDrvCtrl->pMiiDev,
        pDrvCtrl->pMediaList->endMediaListDefault);
    if (rc != OK)
        {
        return ERROR;
        }

    END_FLAGS_SET(pEnd, (IFF_UP | IFF_RUNNING));

    pDrvCtrl->jobQueue = netJobQueueId;
    QJOB_SET_PRI(&pDrvCtrl->intJob, NET_TASK_QJOB_PRI);
    pDrvCtrl->intJob.func = xgmacTxRxIntHandle;

    xgmacHwInit(pDrvCtrl);

    (void) vxbIntEnable(pDrvCtrl->pDev, pDrvCtrl->macIntRes);
    for (i = 0; i < XGMAC_NUM_CHNLS; i++)
        {
        (void) vxbIntEnable(pDrvCtrl->pDev, pDrvCtrl->txIntRes[i]);
        (void) vxbIntEnable(pDrvCtrl->pDev, pDrvCtrl->rxIntRes[i]);
        }

exit:
    (void) END_TX_SEM_GIVE(pEnd);
    (void) semGive(pDrvCtrl->devSem);

    return rc;
    }

/*******************************************************************************
*
* xgmacStop - stop the device
*
* This function undoes the effects of xgmacStart(). The device is
* shut down and all resources are released. Note that the shutdown process
* pauses to wait for all pending RX, TX and link event jobs that may have been
* initiated by the interrupt handler to complete. This is done to prevent
* tNetTask from accessing any data that might be released by this routine.
*
* RETURNS: ERROR if device shutdown failed, otherwise OK
*
* ERRNO: N/A
*/

LOCAL STATUS xgmacStop
    (
    END_OBJ * pEnd      /* pointer to end object */
    )
    {
    return ERROR;
    }


#define XGMAC_MTL_QINT_STATUS(x)    (0x00001174 + (0x80 * (x)))
#define XGMAC_RXOVFIS           BIT(16)
#define XGMAC_ABPSIS            BIT(1)
#define XGMAC_TXUNFIS           BIT(0)

#define XGMAC_DMA_CH_INT_EN(x)      (0x00003138 + (0x80 * (x)))
#define XGMAC_NIE           BIT(15)
#define XGMAC_AIE           BIT(14)
#define XGMAC_RBUE          BIT(7)
#define XGMAC_RIE           BIT(6)
#define XGMAC_TBUE          BIT(2)
#define XGMAC_TIE           BIT(0)
#define XGMAC_DMA_INT_DEFAULT_EN    (XGMAC_NIE | XGMAC_AIE | XGMAC_RBUE | \
                    XGMAC_RIE | XGMAC_TIE)
#define XGMAC_DMA_INT_DEFAULT_RX    (XGMAC_RBUE | XGMAC_RIE)
#define XGMAC_DMA_INT_DEFAULT_TX    (XGMAC_TIE)
#define XGMAC_DMA_CH_Rx_WATCHDOG(x) (0x0000313c + (0x80 * (x)))
#define XGMAC_RWT           GENMASK(7, 0)
#define XGMAC_DMA_CH_STATUS(x)      (0x00003160 + (0x80 * (x)))
#define XGMAC_NIS           BIT(15)
#define XGMAC_AIS           BIT(14)
#define XGMAC_FBE           BIT(12)
#define XGMAC_RBU           BIT(7)
#define XGMAC_RI            BIT(6)
#define XGMAC_TBU           BIT(2)
#define XGMAC_TPS           BIT(1)
#define XGMAC_TI            BIT(0)

#define XGMAC_DMA_STATUS_MSK_COMMON (XGMAC_NIS | XGMAC_AIS | XGMAC_FBE)
#define XGMAC_DMA_STATUS_MSK_RX     (XGMAC_RBU | XGMAC_RI | \
                     XGMAC_DMA_STATUS_MSK_COMMON)
#define XGMAC_DMA_STATUS_MSK_TX     (XGMAC_TBU | XGMAC_TPS | XGMAC_TI | \
                     XGMAC_DMA_STATUS_MSK_COMMON)


/*******************************************************************************
*
* xgmacIsr - MAC interrupt handler
*
*/

LOCAL VOID xgmacIsr
    (
    XGMAC_DRV_CTRL* pDrvCtrl
    )
    {
    UINT32 stat;
    UINT32 qStat;
    UINT32 dmaStat;
    UINT32 i;

    dmaStat = CSR_READ_4(pDrvCtrl, XGMAC_DMA_INT_STATUS);

    if ((dmaStat & BIT(17)) != 0)
        {
        /* MAC status */

        stat = CSR_READ_4(pDrvCtrl, XGMAC_INT_STATUS);
        if (stat != 0)
            {
            //logMsg("MAC INT status = 0x%08x\n", stat, 0, 0, 0, 0, 0);
            }
        }


    if ((dmaStat & BIT(16)) != 0)
        {
        /* MTL status */

        stat = CSR_READ_4(pDrvCtrl, XGMAC_MTL_INT_STATUS);
        if (stat != 0)
            {
            for (i = 0; i < 8; i++)
                {
                if ((stat & BIT(i)) != 0)
                    {
                    qStat = CSR_READ_4(pDrvCtrl, XGMAC_MTL_QINT_STATUS(i));
                    logMsg("MTL QINT(%d) status = 0x%08x\n", i, qStat, 0, 0, 0, 0);
                    CSR_WRITE_4(pDrvCtrl, XGMAC_MTL_QINT_STATUS(i), 0xffffffff);
                    }
                }
            }
        }

    /* DMA channel interrupt?  If so, defer to TX/RX handler. */

    if ((dmaStat & 0x000000ff) != 0)
        {
        for (i = 0; i < XGMAC_NUM_CHNLS; i++)
            {
            DMA_CHNL_INFO* pChInfo;
            UINT32 stat;
            pChInfo = &pDrvCtrl->dmaChInfo[i];

            stat = CSR_READ_4(pDrvCtrl, XGMAC_DMA_CH_STATUS(i));
            if (stat != 0)
                {
                CSR_SETBIT_4(pDrvCtrl, XGMAC_DMA_CH_STATUS(i), stat);
                vxAtomicSet(&pChInfo->intStatus, stat);
                if (vxAtomicSet(&pChInfo->intPending, 1) == 0)
                    {
                    (void) jobQueuePost(pDrvCtrl->jobQueue, &pDrvCtrl->intJob);
                    xgmacDmaIntDisable(pDrvCtrl, i);
                    }
                }

            }
        }
    }


/*******************************************************************************
*
* xgmacSend - transmit a packet
*
*/

LOCAL INT32 xgmacSend
    (
    END_OBJ* pEnd,
    M_BLK_ID  pMblk
    )
    {
    XGMAC_DRV_CTRL* pDrvCtrl;
    M_BLK_ID pTmpMblk;
    UINT32 ch;
    UINT32 len;
    INT32 ret;

    pDrvCtrl = (XGMAC_DRV_CTRL*) pEnd;

    if (pDrvCtrl->polling)
        {
        endPoolTupleFree(pMblk);
        return ERROR;
        }

    (void) END_TX_SEM_TAKE(pEnd, WAIT_FOREVER);

    ch = xgmacTxChnlGet(pDrvCtrl, pMblk);

    ret = xgmacEndEncap(pDrvCtrl, pMblk, ch);

    if (ret == ENOSPC)
        {
        pTmpMblk = endPoolTupleGet(pDrvCtrl->endObj.pNetPool);
        if (pTmpMblk == NULL)
            {
            (void) END_TX_SEM_GIVE(pEnd);
            return ERROR;
            }

        len = (UINT32) netMblkToBufCopy(pMblk, mtod (pTmpMblk, char *), NULL);
        pTmpMblk->m_len = pTmpMblk->m_pkthdr.len = (int) len;
        pTmpMblk->m_flags = pMblk->m_flags;
        endPoolTupleFree(pMblk);

        ret = xgmacEndEncap(pDrvCtrl, pTmpMblk, ch);
        }

    (void) END_TX_SEM_GIVE(pEnd);
    return (ret == OK) ? OK : ERROR;
    }


/*******************************************************************************
*
* xgmacEncap - encapsulate an outbound packet in the TX DMA ring
*
* This function sets up a descriptor for a packet transmit operation. With the
* XEMAC ethernet controller, the TX DMA ring consists of descriptors that each
* describe a single packet fragment.  We consume as many descriptors as there
* are mBlks in the outgoing packet, unless the chain is too long.  The length
* is limited by the number of DMA segments we want to allow in a given DMA map.
* If there are too many segments, this routine will fail, and the caller must
* coalesce the data into fewer buffers and try again.
*
* This routine will also fail if there aren't enough free descriptors available
* in the ring, in which case the caller must defer the transmission until more
* descriptors are completed by the chip.
*
* RETURNS: ENOSPC if there are too many fragments in the packet, EAGAIN
* if the DMA ring is full, otherwise OK.
*
*/

LOCAL INT32 xgmacEndEncap
    (
    XGMAC_DRV_CTRL* pDrvCtrl,
    M_BLK_ID pMblk,
    UINT32 ch
    )
    {
    DMA_CHNL_INFO* pChInfo;
    DMA_DESC* pFirstDesc;
    DMA_DESC* pDesc;
    DMA_DESC_INFO* pFirstDescInfo;
    DMA_DESC_INFO* pDescInfo;
    VXB_DMA_MAP_ID pMap;
    PHYS_ADDR physAddr;
    UINT32 tail;
    UINT32 frameLen;
    INT32 i;
    STATUS rc;

    pChInfo = &pDrvCtrl->dmaChInfo[ch];
    tail = pChInfo->txDescRingTail;

    /* Ring full? */

    if (pChInfo->txDescInfo[tail].pMblk != NULL)
        {
        return EAGAIN;
        }

    /*
     * Load the DMA map to build the segment list.
     * This will fail if there are too many segments.
     */

    pFirstDesc = &pChInfo->pTxDesc[tail];
    pFirstDescInfo = &pChInfo->txDescInfo[tail];
    pMap = pFirstDescInfo->map;
    rc = vxbDmaBufMapMblkLoad(pDrvCtrl->pDev, pDrvCtrl->mBlkDmaTag, pMap, pMblk, 0);
    if (rc != OK)
        {
        (void) vxbDmaBufMapUnload(pDrvCtrl->mBlkDmaTag, pMap);
        return (ENOSPC);
        }

    frameLen = 0;
    for (i = 0; i < pMap->nFrags; i++)
        {
        UINT32 val;
        pDesc = &pChInfo->pTxDesc[tail];
        pDescInfo = &pChInfo->txDescInfo[tail];

        val = (UINT32) pMap->fragList[i].fragLen;
        frameLen += val;
        physAddr = pMap->fragList[i].frag;
        pDesc->desc0 = VXB_ADDR_LOW32(physAddr);
        pDesc->desc1 = VXB_ADDR_HIGH32(physAddr);
        pDesc->desc2 = (UINT32) val & XGMAC_TDES2_B1L;
        if (pDesc == pFirstDesc)
            {
            pDesc->desc3 = XGMAC_TDES3_FD;
            }
        else
            {
            pDesc->desc3 = XGMAC_TDES3_OWN;
            }

        pChInfo->txDescRingFree--;
        tail = (tail + 1) & (pDrvCtrl->txDescRingSize - 1);
        }

    /* Update last descriptor */

    pDesc->desc2 |= XGMAC_TDES2_IOC;
    pDesc->desc3 |= XGMAC_TDES3_LD;

    /* Update first descriptor */

    pFirstDesc->desc3 |= frameLen;
    VX_MEM_BARRIER_W();
    pFirstDesc->desc3 |= XGMAC_TDES3_OWN;

    /*
     * Ensure that the map for this transmission
     * is placed at the array index of the last descriptor
     * in this chain.  (Swap last and first DMA maps.)
     */

    pFirstDescInfo->map = pDescInfo->map;
    pDescInfo->map = pMap;

    /* save the mBlk for later */

    pDescInfo->pMblk = pMblk;

    pChInfo->txDescRingTail = tail;

    /* Copy to bounce buffer if necessary and flush the buffers. */

    (void) vxbDmaBufSync (pDrvCtrl->pDev, pDrvCtrl->mBlkDmaTag, pMap,
               _VXB_DMABUFSYNC_DMA_PREWRITE);

    CSR_WRITE_4(pDrvCtrl, XGMAC_DMA_CH_TxDESC_TAIL_LPTR(ch),
        (VXB_ADDR_LOW32(pChInfo->txDescPhysAddr) + (pChInfo->txDescRingTail - 1) * sizeof(DMA_DESC)));
    return OK;
    }


LOCAL UINT32 xgmacTxChnlGet
    (
    XGMAC_DRV_CTRL* pDrvCtrl,
    M_BLK_ID pMblk
    )
    {
    UINT32 index = 0;

    if (pDrvCtrl->endCaps.cap_available & IFCAP_TIME_GATED_QBV)
        {
        switch (pMblk->mBlkPktHdr.traffic_class)
            {
            case PKT_TRAFFIC_CLASS0:
                index = 0;
                break;
            case PKT_TRAFFIC_CLASS1:
                index = 1;
                break;
            case PKT_TRAFFIC_CLASS2:
                index = 2;
                break;
            case PKT_TRAFFIC_CLASS3:
                index = 3;
                break;
            case PKT_TRAFFIC_CLASS4:
                index = 4;
                break;
            case PKT_TRAFFIC_CLASS5:
                index = 5;
                break;
            case PKT_TRAFFIC_CLASS6:
                index = 6;
                break;
            case PKT_TRAFFIC_CLASS7:
                index = 7;
                break;
            default:
                index = 0;
                break;
            }
        }
    else
        {
        index = _WRS_CPU_INDEX_GET();
        }

    return index;
    }

/*******************************************************************************
*
* xgmacMiiRead - miiBus miiRead method
*
* RETURNS: OK or ERROR
*
*/

LOCAL STATUS xgmacMiiRead
    (
    VXB_DEV_ID pDev,
    UINT8      phyAddr,
    UINT8      regAddr,
    UINT16 *   pRegVal
    )
    {
    XGMAC_DRV_CTRL* pDrvCtrl;
    UINT32          addrVal;
    UINT32          dataVal;
    STATUS          rc;

    pDrvCtrl = vxbDevSoftcGet(pDev);

    rc = semTake(pDrvCtrl->devSem, WAIT_FOREVER);
    if (rc != OK)
        {
        return rc;
        }

    /* Indicates that Port 0 is connected to a PHY that only supports clause 22. */

    dataVal = CSR_READ_4(pDrvCtrl, XGMAC_MDIO_C22P);
    dataVal &= ~MII_XGMAC_C22P_MASK;
    dataVal |= BIT(phyAddr);
    CSR_WRITE_4(pDrvCtrl, XGMAC_MDIO_C22P, dataVal);

    xgmacMdioBusyWait(pDrvCtrl);

    /* Set address */

    addrVal = ((UINT32) phyAddr << MII_XGMAC_PA_SHIFT) | (UINT32) regAddr;
    CSR_WRITE_4 (pDrvCtrl, XGMAC_MDIO_ADDR, addrVal);

    /* Set data */

    dataVal = XGMAC_MDIO_DATA_READ | XGMAC_MDIO_DATA_SADDR |
        XGMAC_MDIO_DATA_SBUSY;
    dataVal |= (pDrvCtrl->mdioCrVal << 19);
    CSR_WRITE_4(pDrvCtrl, XGMAC_MDIO_DATA, dataVal);

    xgmacMdioBusyWait(pDrvCtrl);

    dataVal = CSR_READ_4(pDrvCtrl, XGMAC_MDIO_DATA);
    *pRegVal = (UINT16)(dataVal & 0x0000ffff);
    XGMAC_DBG(DBG_MDIO, "%s: phy = %d, reg = %d, val = 0x%08x\n",
        __FUNCTION__, phyAddr, regAddr, *pRegVal);

    (void) semGive(pDrvCtrl->devSem);
    return OK;
    }


/*******************************************************************************
*
* xgmacMiiWrite - miiBus miiWrite method
*
* RETURNS: OK or ERROR
*
*/

LOCAL STATUS xgmacMiiWrite
    (
    VXB_DEV_ID  pDev,
    UINT8       phyAddr,
    UINT8       regAddr,
    UINT16      regVal
    )
    {
    XGMAC_DRV_CTRL* pDrvCtrl;
    UINT32          addrVal;
    UINT32          dataVal;
    STATUS          rc;

    pDrvCtrl = vxbDevSoftcGet(pDev);

    rc = semTake(pDrvCtrl->devSem, WAIT_FOREVER);
    if (rc != OK)
        {
        return rc;
        }

    /* Indicates that Port 0 is connected to a PHY that only supports Clause 22. */

    dataVal = CSR_READ_4(pDrvCtrl, XGMAC_MDIO_C22P);
    dataVal &= ~MII_XGMAC_C22P_MASK;
    dataVal |= BIT(phyAddr);
    CSR_WRITE_4(pDrvCtrl, XGMAC_MDIO_C22P, dataVal);

    /* Set address */

    addrVal = ((UINT32) phyAddr << MII_XGMAC_PA_SHIFT) | (UINT32) regAddr;
    CSR_WRITE_4(pDrvCtrl, XGMAC_MDIO_ADDR, addrVal);

    xgmacMdioBusyWait(pDrvCtrl);

    /* Set data */

    dataVal = XGMAC_MDIO_DATA_WRITE | XGMAC_MDIO_DATA_SADDR |
        XGMAC_MDIO_DATA_SBUSY;
    dataVal |= (pDrvCtrl->mdioCrVal << 19);
    CSR_WRITE_4(pDrvCtrl, XGMAC_MDIO_DATA, dataVal);

    xgmacMdioBusyWait(pDrvCtrl);

    (void)semGive(pDrvCtrl->devSem);

    return OK;
    }


/*******************************************************************************
*
* xgmacMediaAttach - MII media attach method
*
* This function implements the MII media attach method. It will be invoked
* as a callback to provide a handle to the PHY device attached to this
* network interface.
*
* RETURNS: OR or ERROR
*
*/

LOCAL STATUS xgmacMediaAttach
    (
    VXB_DEV_ID pDev,
    VXB_DEV_ID miiDev
    )
    {
    XGMAC_DRV_CTRL * pDrvCtrl = NULL;

    pDrvCtrl = (XGMAC_DRV_CTRL *) vxbDevSoftcGet(pDev);
    if (pDrvCtrl == NULL)
        {
        return (ERROR);
        }

    (void) semTake (pDrvCtrl->devSem, WAIT_FOREVER);

    pDrvCtrl->pMiiDev = miiDev;

    (void) semGive(pDrvCtrl->devSem);

    return (OK);
    }


/*****************************************************************************
*
* xgmacMediaUpdate - miiBus miiMediaUpdate method
*
* RETURNS: OK or ERROR
*
* ERRNO: N/A
*/

LOCAL STATUS xgmacMediaUpdate
    (
    VXB_DEV_ID pDev
    )
    {
    XGMAC_DRV_CTRL* pDrvCtrl;
    UINT32          oldStatus;
    UINT32          oldMode;
    UINT32          val;
    STATUS          rc;

    pDrvCtrl = vxbDevSoftcGet(pDev);

    (void) semTake(pDrvCtrl->devSem, WAIT_FOREVER);

    if ((pDrvCtrl->endObj.flags & IFF_UP) == 0)
        {
        rc = OK;
        goto exit;
        }

    if (pDrvCtrl->pMiiDev == NULL)
        {
        rc = ERROR;
        goto exit;
        }

    oldStatus = pDrvCtrl->miiStatus;
    oldMode  = pDrvCtrl->miiMode;
    rc = miiBusModeGet(pDrvCtrl->pMiiDev, &pDrvCtrl->miiMode,
        &pDrvCtrl->miiStatus);
    if (rc != OK)
        {
        rc = ERROR;
        goto exit;
        }

    /* Anything change?  If not, return OK */

    if ((oldStatus == pDrvCtrl->miiStatus) &&
        (oldMode == pDrvCtrl->miiMode))
        {
        rc = OK;
        goto exit;
        }

    switch (IFM_SUBTYPE(pDrvCtrl->miiMode))
        {
        case IFM_1000_T:
            pDrvCtrl->endObj.mib2Tbl.ifSpeed = 1000000000;
            break;
        case IFM_100_TX:
            pDrvCtrl->endObj.mib2Tbl.ifSpeed = 100000000;
            break;
        case IFM_10_T:
            pDrvCtrl->endObj.mib2Tbl.ifSpeed = 10000000;
            break;
        default:
            pDrvCtrl->endObj.mib2Tbl.ifSpeed = 0;
            break;
        }

    /* TODO - IFM_SUBTYPE reports incorrectly on Simics so
     *  force speed to 1G
     */

    pDrvCtrl->endObj.mib2Tbl.ifSpeed = 1000000000;

    if (pDrvCtrl->endObj.pMib2Tbl != NULL)
        {
        pDrvCtrl->endObj.pMib2Tbl->m2Data.mibIfTbl.ifSpeed =
            pDrvCtrl->endObj.mib2Tbl.ifSpeed;
        }

    if (((pDrvCtrl->miiStatus & IFM_ACTIVE) == IFM_ACTIVE) &&
        ((oldStatus & IFM_ACTIVE) == 0))
        {
        /* TODO - IFM_SUBTYPE reports incorrectly on Simics so
         *  force speed to 1G
         */
        val = CSR_READ_4(pDrvCtrl, XGMAC_TX_CONFIG);
        val &= ~XGMAC_CONFIG_SS_MASK;
        val |= XGMAC_CONFIG_SS_1000_GMII;
        val |= XGMAC_CONFIG_TE;
        CSR_WRITE_4(pDrvCtrl, XGMAC_TX_CONFIG, val);
        (void) jobQueueStdPost(pDrvCtrl->jobQueue, NET_TASK_QJOB_PRI,
            (VOIDFUNCPTR)muxLinkUpNotify, &pDrvCtrl->endObj, NULL, NULL, NULL, NULL);
        }

    else if (((pDrvCtrl->miiStatus & IFM_ACTIVE) == 0) &&
        ((oldStatus & IFM_ACTIVE) == IFM_ACTIVE))
        {
        /* Disable TX */
        val = CSR_READ_4(pDrvCtrl, XGMAC_TX_CONFIG);
        val &= ~XGMAC_CONFIG_SS_MASK;
        val &= ~XGMAC_CONFIG_TE;
        CSR_WRITE_4(pDrvCtrl, XGMAC_TX_CONFIG, val);
        (void) jobQueueStdPost(pDrvCtrl->jobQueue, NET_TASK_QJOB_PRI,
            (VOIDFUNCPTR)muxLinkDownNotify, &pDrvCtrl->endObj, NULL, NULL, NULL, NULL);
        }

exit:
    (void) semGive(pDrvCtrl->devSem);
    return rc;
    }


/******************************************************************************
*
* xgmacMdioBusyWait - Wait until MDIO is not busy.
*
* ERRNO: N/A
*/

LOCAL VOID xgmacMdioBusyWait
    (
    XGMAC_DRV_CTRL* pDrvCtrl
    )
    {
    UINT32 val;
    for (;;)
        {
        val = CSR_READ_4(pDrvCtrl, XGMAC_MDIO_DATA);
        if ((val & XGMAC_MDIO_DATA_SBUSY) == 0)
            {
            break;
            }
        }
    }


/*******************************************************************************
*
* xgmacTxRxIntHandle - process transmit and received interrupts.
*
* This function is scheduled by the ISR to run in the context of tNetTask
* whenever an RX/TX interrupt is received.
*
*/

LOCAL VOID xgmacTxRxIntHandle
    (
    void* pArg
    )
    {
    XGMAC_DRV_CTRL*  pDrvCtrl;
    DMA_CHNL_INFO* pChInfo;
    UINT32 i;

    pDrvCtrl = member_to_object(pArg, XGMAC_DRV_CTRL, intJob);

    for (i = 0; i < XGMAC_NUM_CHNLS; i++)
        {
        pChInfo = &pDrvCtrl->dmaChInfo[i];

        if ((pChInfo->intStatus & XGMAC_RIE) != 0)
            {
            xgmacRxHandle(pDrvCtrl, i);
            }

        if ((pChInfo->intStatus & XGMAC_TIE) != 0)
            {
            xgmacTxHandle(pDrvCtrl, i);
            }
        }

    for (i = 0; i < XGMAC_NUM_CHNLS; i++)
        {
        pChInfo = &pDrvCtrl->dmaChInfo[i];

        if (pChInfo->intStatus != 0)
            {
            vxAtomicSet(&pChInfo->intStatus, 0);
            vxAtomicSet(&pChInfo->intPending, 0);
            xgmacDmaIntEnable(pDrvCtrl, i);
            }
        }
    }


/*******************************************************************************
*
* xgmacRxHandle - process received frames
*
*/

static void xgmacRxHandle
    (
    XGMAC_DRV_CTRL * pDrvCtrl,
    UINT32 ch
    )
    {
    M_BLK_ID pMblk;
    M_BLK_ID pNewMblk;
    DMA_CHNL_INFO* pChInfo;
    DMA_DESC* pDesc;
    DMA_DESC_INFO* pDescInfo;
    VXB_DMA_MAP_ID pMap;
    UINT32 current;
    UINT32 count;
    INT32 len;
    PHYS_ADDR physAddr;

    pChInfo = &pDrvCtrl->dmaChInfo[ch];
    current = pChInfo->rxDescRingCurrent;

    for (count = 0;; count++)
        {
        pDesc = &pChInfo->pRxDesc[current];
        pDescInfo = &pChInfo->rxDescInfo[current];

        if ((pDesc->desc3 & XGMAC_RDES3_OWN) == XGMAC_RDES3_OWN)
            {
            if (count == 0)
                {
                // TODO: Simics bug?  It seem that for every other RX
                // interrupt, the descriptor isn't updated until ~100 us
                // after the interrupt.  Here we delay and retry.
                //printf("count = %d, current = %d, desc3 = 0x%08x\n",
                //   count, current, pDesc->desc3);
                //
                vxbUsDelay(100);
                continue;
                }
            else
                {
                break;
                }
            }

        len = pDesc->desc3 & XGMAC_RDES3_PL;

        pNewMblk = endPoolTupleGet(pDrvCtrl->endObj.pNetPool);
        if (pNewMblk == NULL)
            {
            printf("endPoolTupleGet() error\n");
            break;
            }

        pMblk = pDescInfo->pMblk;
        pDescInfo->pMblk = pNewMblk;

         /* Sync the packet buffer and unload the map. */

        pMap = pDescInfo->map;
        (void) vxbDmaBufMapSync(pDrvCtrl->pDev, pDrvCtrl->mBlkDmaTag,
            pMap, 0, (size_t) len, _VXB_DMABUFSYNC_DMA_POSTREAD);
        (void) vxbDmaBufMapUnload(pDrvCtrl->mBlkDmaTag, pMap);

        /* Set the mBlk header up with the frame length. */

        pMblk->m_flags |= (M_PKTHDR | M_EXT);
        pMblk->m_pkthdr.len = pMblk->m_len = len;

        /* Update statistics for counters */

        pDrvCtrl->endStatsCounters.ifInOctets += (unsigned int)pMblk->m_len;
        xgmacRxCounterUpdate(pDrvCtrl, pMblk);

        END_RCV_RTN_CALL(&pDrvCtrl->endObj, pMblk);

        /* Update descriptor with new mBlk */

        (void) vxbDmaBufMapLoad(pDrvCtrl->pDev, pDrvCtrl->mBlkDmaTag, pMap,
            pNewMblk->m_data, (bus_size_t)(pDrvCtrl->mtu), 0);

        (void) vxbDmaBufSync(pDrvCtrl->pDev, pDrvCtrl->mBlkDmaTag, pMap,
            _VXB_DMABUFSYNC_DMA_PREREAD);

        physAddr = pMap->fragList[0].frag;

        pDesc->desc0 = VXB_ADDR_LOW32(physAddr);
        pDesc->desc1 = VXB_ADDR_HIGH32(physAddr);
        pDesc->desc2 = 0;
        VX_MEM_BARRIER_W();
        pDesc->desc3 = XGMAC_RDES3_OWN | XGMAC_RDES3_IOC;

        /* Update RX tail like u-boot does */

        if ((current & (DESCS_PER_CACHE_LINE - 1)) == (DESCS_PER_CACHE_LINE - 1))
            {
            pChInfo->rxDescRingTail = current;
            CSR_WRITE_4(pDrvCtrl, XGMAC_DMA_CH_RxDESC_TAIL_LPTR(ch),
                (VXB_ADDR_LOW32(pChInfo->rxDescPhysAddr) + pChInfo->rxDescRingTail * sizeof(DMA_DESC)));
            }

        current = (current + 1) & (pDrvCtrl->rxDescRingSize - 1);
        }

        pChInfo->rxDescRingCurrent = current;
    }


/*****************************************************************************
*
* xgmacTxHandle - process transmitted frames
*
*/
static void xgmacTxHandle
    (
    XGMAC_DRV_CTRL * pDrvCtrl,
    UINT32 ch
    )
    {
    M_BLK_ID pMblk;
    UINT32 current;
    UINT32 tail;
    DMA_CHNL_INFO* pChInfo;
    DMA_DESC* pDesc;
    DMA_DESC_INFO* pDescInfo;

    pChInfo = &pDrvCtrl->dmaChInfo[ch];
    current = pChInfo->txDescRingCurrent;
    tail = pChInfo->txDescRingTail;

    (void) END_TX_SEM_TAKE(&pDrvCtrl->endObj, WAIT_FOREVER);

    for (;;)
        {
        pDesc = &pChInfo->pTxDesc[current];
        pDescInfo = &pChInfo->txDescInfo[current];

        if ((pDesc->desc3 & XGMAC_TDES3_OWN) != 0)
            {
            break;
            }

        if ((pDesc->desc3 & XGMAC_TDES3_LD) == XGMAC_TDES3_LD)
            {
            pMblk = pDescInfo->pMblk;
            if (pMblk == NULL)
                {
                XGMAC_DBG(DBG_ERR, "%s ERROR: M_BLK_ID is NULL\n", __FUNCTION__);
                break;
                }
            pDescInfo->pMblk = NULL;

            pDrvCtrl->endStatsCounters.ifOutOctets += (unsigned int) pMblk->m_len;
            xgmacTxCounterUpdate(pDrvCtrl, pMblk);

            (void) vxbDmaBufMapUnload(pDrvCtrl->mBlkDmaTag, pDescInfo->map);
            netMblkClChainFree(pMblk);
            }

        pDesc->desc0 = 0;
        pDesc->desc1 = 0;
        pDesc->desc2 = 0;
        pDesc->desc3 = 0;
        pChInfo->txDescRingFree++;

        current = (current + 1) & (pDrvCtrl->txDescRingSize - 1);

        if (current == tail)
            {
            break;
            }
        }

    pChInfo->txDescRingCurrent = current;
    (void) END_TX_SEM_GIVE(&pDrvCtrl->endObj);
    }


/*******************************************************************************
*
* xgmacTxCounterUpdate - Update TX statistics
*
*/

LOCAL VOID xgmacTxCounterUpdate
    (
    XGMAC_DRV_CTRL *    pDrvCtrl,
    M_BLK_ID            pMblk
    )
    {
    if (((uint8_t)pMblk->m_data[0] == 0xFF) &&
        ((uint8_t)pMblk->m_data[1] == 0xFF) &&
        ((uint8_t)pMblk->m_data[2] == 0xFF) &&
        ((uint8_t)pMblk->m_data[3] == 0xFF) &&
        ((uint8_t)pMblk->m_data[4] == 0xFF) &&
        ((uint8_t)pMblk->m_data[5] == 0xFF))
        {
        pDrvCtrl->endStatsCounters.ifOutBroadcastPkts++;
        }
    else if ((uint8_t)pMblk->m_data[0] & 0x1)
        {
        pDrvCtrl->endStatsCounters.ifOutMulticastPkts++;
        }
    else
        {
        pDrvCtrl->endStatsCounters.ifOutUcastPkts++;
        }
    }


/*************************************************************************
*
* xgmacRxCounterUpdate - Update RX statistics
*
*/

LOCAL VOID xgmacRxCounterUpdate
    (
    XGMAC_DRV_CTRL *    pDrvCtrl,
    M_BLK_ID            pMblk
    )
    {
    if (((uint8_t)pMblk->m_data[0] == 0xFF) &&
        ((uint8_t)pMblk->m_data[1] == 0xFF) &&
        ((uint8_t)pMblk->m_data[2] == 0xFF) &&
        ((uint8_t)pMblk->m_data[3] == 0xFF) &&
        ((uint8_t)pMblk->m_data[4] == 0xFF) &&
        ((uint8_t)pMblk->m_data[5] == 0xFF))
        {
        pDrvCtrl->endStatsCounters.ifInBroadcastPkts++;
        }
    else if ((uint8_t)pMblk->m_data[0] & 0x1)
        {
        pDrvCtrl->endStatsCounters.ifInMulticastPkts++;
        }
    else
        {
        pDrvCtrl->endStatsCounters.ifInUcastPkts++;
        }
    }


/*******************************************************************************
*
* xgmacEndPollSend - polled mode transmit routine
*
* This function is similar to the xgmacEndSend() routine shown above,
* except it performs transmissions synchronously with interrupts disabled.
* After the transmission is initiated, the routine will poll the state of the
* TX status register associated with the current slot until transmission
* completed. If transmission times out, this routine will return ERROR.
*
* RETURNS: OK, EAGAIN, or ERROR
*
* ERRNO: N/A
*/

LOCAL STATUS xgmacPollSend
    (
    END_OBJ* pEnd,
    M_BLK_ID pMblk
    )
    {
    XGMAC_DRV_CTRL* pDrvCtrl;
    DMA_CHNL_INFO* pChInfo;
    DMA_DESC* pDesc;
    DMA_DESC_INFO* pDescInfo;
    UINT32 current;
    UINT32 tail;
    UINT32 ch;
    INT32 ret;

    pDrvCtrl = (XGMAC_DRV_CTRL *) pEnd;
    if (!pDrvCtrl->polling)
        {
        return ERROR;
        }

    ch = xgmacTxChnlGet(pDrvCtrl, pMblk);

    ret = xgmacEndEncap(pDrvCtrl, pMblk, ch);
    if (ret != OK)
        {
        return EAGAIN;
        }

    pChInfo = &pDrvCtrl->dmaChInfo[ch];
    current = pChInfo->txDescRingCurrent;
    tail = pChInfo->txDescRingTail;

    /* Wait for TX to complete */

    for (;;)
        {
        pDesc = &pChInfo->pTxDesc[current];
        pDescInfo = &pChInfo->txDescInfo[current];

        if ((pDesc->desc3 & XGMAC_TDES3_OWN) != 0)
            {
            break;
            }

        if ((pDesc->desc3 & XGMAC_TDES3_LD) == XGMAC_TDES3_LD)
            {
            pMblk = pDescInfo->pMblk;
            pDescInfo->pMblk = NULL;

            pDrvCtrl->endStatsCounters.ifOutOctets += (unsigned int) pMblk->m_len;
            xgmacTxCounterUpdate(pDrvCtrl, pMblk);
            pDesc->desc0 = 0;
            pDesc->desc1 = 0;
            pDesc->desc2 = 0;
            pDesc->desc3 = 0;
            pChInfo->txDescRingFree++;
            (void) vxbDmaBufMapUnload(pDrvCtrl->mBlkDmaTag, pDescInfo->map);
            current = (current + 1) & (pDrvCtrl->txDescRingSize - 1);
            break;
            }
        else
            {
            current = (current + 1) & (pDrvCtrl->txDescRingSize - 1);
            }

        if (current == tail)
            {
            break;
            }
        }

    pChInfo->txDescRingCurrent = current;
    return OK;
    }


/*******************************************************************************
*
* xgmacEndPollReceive - polled mode receive routine
*
* This function receive a packet in polled mode, with interrupts disabled.
* It's similar in operation to the xgmacEndRxHandle() routine, except it
* doesn't process more than one packet at a time and does not load out
* buffers. Instead, the caller supplied an mBlk tuple into which this
* function will place the received packet.
*
* If no packet is available, this routine will return EAGAIN. If the
* supplied mBlk is too small to contain the received frame, the routine
* will return ERROR.
*
* RETURNS: OK, EAGAIN, or ERROR
*
*/

LOCAL STATUS xgmacPollReceive
    (
    END_OBJ* pEnd,
    M_BLK_ID pMblk
    )
    {
    XGMAC_DRV_CTRL* pDrvCtrl;
    M_BLK_ID pTmpMblk;
    DMA_CHNL_INFO* pChInfo;
    DMA_DESC* pDesc;
    DMA_DESC_INFO* pDescInfo;
    VXB_DMA_MAP_ID pMap;
    UINT32 current;
    INT32 len;

    pDrvCtrl = (XGMAC_DRV_CTRL*) pEnd;

    if (!pDrvCtrl->polling)
        {
        return ERROR;
        }

    pChInfo = &pDrvCtrl->dmaChInfo[0];
    current = pChInfo->rxDescRingCurrent;

    pDesc = &pChInfo->pRxDesc[current];
    pDescInfo = &pChInfo->rxDescInfo[current];

    if ((pDesc->desc3 & XGMAC_RDES3_OWN) != 0)
        {
        return EAGAIN;
        }

    pTmpMblk = pDescInfo->pMblk;

    len = pDesc->desc3 & XGMAC_RDES3_PL;

    pMap = pDescInfo->map;
    (void) vxbDmaBufMapSync(pDrvCtrl->pDev, pDrvCtrl->mBlkDmaTag,
        pMap, 0, (size_t) len, _VXB_DMABUFSYNC_DMA_POSTREAD);
    (void) vxbDmaBufMapUnload(pDrvCtrl->mBlkDmaTag, pMap);

    pDrvCtrl->endStatsCounters.ifInOctets += (unsigned int) pTmpMblk->m_len;
    xgmacRxCounterUpdate(pDrvCtrl, pTmpMblk);

    pDesc->desc2 = 0;
    VX_MEM_BARRIER_W();
    pDesc->desc3 = XGMAC_RDES3_OWN | XGMAC_RDES3_IOC;

    pMblk->m_flags |= (M_PKTHDR | M_EXT);
    pMblk->m_pkthdr.len = pMblk->m_len = len;
    bcopy((char *)pTmpMblk->m_data, (char *)pMblk->m_data, (size_t)pMblk->m_len);
    XGMAC_DBG(DBG_RX, "%s cur %d M_BLK_ID %p len %d\n", __FUNCTION__, cur, pMblk, len);
    current = (current + 1) & (pDrvCtrl->rxDescRingSize - 1);
    pChInfo->rxDescRingCurrent = current;
    return OK;
    }


VOID xgmacRxRingShow(VOID)
    {
    XGMAC_DRV_CTRL* pDrvCtrl = gDrvCtrl;
    DMA_CHNL_INFO* pChInfo;
    DMA_DESC* pDesc;
    UINT32 i;

    pChInfo = &pDrvCtrl->dmaChInfo[0];
    for (i = 0; i < pDrvCtrl->rxDescRingSize; i++)
        {
        pDesc = &pChInfo->pRxDesc[i];
        printf("%d: desc3: 0x%08x\n", i, pDesc->desc3);
        }
    }
