#!/bin/bash -x

HOME=$1     # 1st argument is user home directory
WS=$2       # 2nd argument must be workspace directory
IMG=${WS}/vxworks7.img   # Filename for the disk image
SIZE=32     # Size in MB

# Create a disk image of appropriate size
dd if=/dev/zero of=${IMG} bs=1M count=${SIZE} status=none

# Mount the disk image on a loop device
LOOP=$(losetup -fL --show ${IMG})
test -z ${LOOP} && { echo "ERROR: loop device not allocated"; exit 1; }

# Create the partition table
sudo parted -a opt -s ${LOOP} \
    mklabel msdos \
    mkpart primary fat32 4M 100%

# Re-read the partition table
sudo partprobe ${LOOP}

# Format the partitions
mkfs.vfat ${LOOP}p1

# Close the loop device for saving the changes
losetup -d ${LOOP}

# Mount the 1st partition
LOOP=$(losetup -fL --show ${IMG})
sudo partprobe ${LOOP}
sudo mount ${LOOP}p1 mnt

# Copy files
echo copying files...
cp -v ${HOME}/usa-itl299641_agies-code/binaries/u-boot.itb mnt/
cp -v ${HOME}/usa-itl299641_agies-code/binaries/vxboot.scr mnt/
cp -v ${WS}/vip_itl_agilex5/default/uVxWorks mnt/
cp -v ${WS}/vip_itl_agilex5/default/socfpga_agilex5_socdk.dtb mnt/

# Unmount partition
sudo umount mnt
losetup -d ${LOOP}
