#
# Create VIP
#
import argparse
import os
import subprocess

parser = argparse.ArgumentParser(description="Create VIP")
parser.add_argument("--verbose", action='store_true', help="Verbose output")
parser.add_argument("--debug", action='store_true', help="Add debug components")
parser.add_argument("--vxtest", action='store_true', help="VXTEST")
args = parser.parse_args()

verbose = args.verbose
debug = args.debug
vxtest = args.vxtest

bsp="itl_agilex5"
vsb="vsb_itl_agilex5"
vip="vip_itl_agilex5"

prj_dir=f"./{vip}"
prj_file=f"{prj_dir}/{vip}.wpj"

bundles = \
    [
    "BUNDLE_STANDALONE_SHELL",
    ]

components = \
    [
    "INCLUDE_IPATTACH",
    "INCLUDE_IFCONFIG",
    "INCLUDE_PING",
    "INCLUDE_AUX_CLK",
    "INCLUDE_DOSFS",
    "INCLUDE_RAWFS",
    "INCLUDE_ERF",
    "INCLUDE_DEVICE_MANAGER",
    "INCLUDE_FS_EVENT_UTIL",
    "INCLUDE_FS_MONITOR",
    "INCLUDE_XBD",
    "INCLUDE_XBD_BLK_DEV",
    "INCLUDE_DOSFS_DIR_FIXED",
    "INCLUDE_DOSFS_DIR_VFAT",
    "INCLUDE_DOSFS_FAT",
    "INCLUDE_DOSFS_FMT",
    "INCLUDE_DOSFS_CHKDSK",
    "INCLUDE_DOSFS_MAIN",
    "INCLUDE_SHELL_INTERP_CMD",
    "INCLUDE_IPTFTP_CLIENT_CMD",
    ]

if debug:
    components += \
    [
    "INCLUDE_VXBUS_SHOW",
    "INCLUDE_ADR_SPACE_SHOW",
    "INCLUDE_VM_SHOW",
    "INCLUDE_SHOW_ROUTINES",
    "INCLUDE_DEBUG_KPRINTF",
    "INCLUDE_USB_SHOW"
    ]

if vxtest:
    components += \
    [
    "INCLUDE_TM_BASIS",
    ]

parameters = \
    [
    f'vxprj vip parameter setstring {vip} MTD_CFG_STR "\$hpnfc0#0:0x0,0x3fc,/ftl0#0,0"',
    f'vxprj vip parameter set {vip} FTL_AUTO_MOUNT FALSE',
    f'vxprj vip parameter setstring {vip} VXBFLASH_CFG_STR "\$qspiflash0#0:0x0,0x10000000,0x0c000000,rfa0"',
    ]

if not os.environ.get("WIND_BASE"):
    print("Error: WIND_BASE is not set.")
    exit()

if not os.environ.get("WIND_WRTOOL_WORKSPACE"):
    print("Error: WIND_WRTOOL_WORKSPACE is not set.")
    exit()

prj_cmd = f"vxprj vip create -force -vsb {vsb} {bsp} {vip}"
print("Creating VIP...")
if verbose:
    print(prj_cmd)

response = subprocess.run(prj_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
if response.returncode != 0:
    print(response.stdout)
    print(response.stderr)
    exit()

if len(bundles) > 0:
    bundles_str = " ".join(bundles)

    prj_cmd = f"vxprj vip bundle add {vip} {bundles_str}"
    print("Adding bundles...")
    if verbose:
        print(prj_cmd)

response = subprocess.run(prj_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
if response.returncode != 0:
    print(response.stdout)
    print(response.stderr)
    exit()

components_str = " ".join(components)
prj_cmd = f"vxprj vip component add {vip} {components_str}"
print("Adding components...")
if verbose:
    print(prj_cmd)

proc = subprocess.run(prj_cmd, capture_output=True, text=True, shell=True)
if proc.returncode != 0:
    print(proc)
    exit()

# Set parameters

print("Setting parameters...")
for prj_cmd in parameters:
    if verbose:
        print(prj_cmd)
    proc = subprocess.run(prj_cmd, capture_output=True, text=True, shell=True)
    if proc.returncode != 0:
        print(proc)
        exit()
