#
# Create VSB
#
import argparse
import os
import subprocess

parser = argparse.ArgumentParser(description="Create VSB")
parser.add_argument("--verbose", action='store_true', help="Verbose output")
parser.add_argument("--debug", action='store_true', help="Debug")
parser.add_argument("--vxtest", action='store_true', help="VXTEST")
args = parser.parse_args()

bsp="itl_agilex5"
vsb="vsb_itl_agilex5"

verbose = args.verbose
debug = args.debug
vxtest = args.vxtest

layers = \
    [
    "USB"
    ]

if not os.environ.get("WIND_BASE"):
    print("Error: WIND_BASE is not set.")
    exit()

if not os.environ.get("WIND_WRTOOL_WORKSPACE"):
    print("Error: WIND_WRTOOL_WORKSPACE is not set.")
    exit()

prj_cmd = f"vxprj vsb create -S -force -bsp {bsp} {vsb}"
print("Creating VSB...")
if verbose:
    print(prj_cmd)

proc = subprocess.run(prj_cmd, capture_output=True, text=True, shell=True)
if proc.returncode != 0:
    print(proc)
    exit()

if len(layers) > 0:
    layers_str = " ".join(layers)
    prj_cmd = f"vxprj vsb add {layers_str} {vsb}"
    print("Adding layers to VSB...")
    if verbose:
        print(prj_cmd)

    proc = subprocess.run(prj_cmd, capture_output=True, text=True, shell=True)
    if proc.returncode != 0:
        print(proc)
        exit()

if debug:
    prj_cmd = f"vxprj vsb config -w -add _WRS_CONFIG_DEBUG_FLAG=y {vsb}"
    print("Adding debug to VSB...")
    if verbose:
        print(prj_cmd)
    proc = subprocess.run(prj_cmd, capture_output=True, text=True, shell=True)
    if proc.returncode != 0:
        print(proc)
        exit()

if vxtest:
    prj_cmd = f"vxprj vsb config -w -add _WRS_CONFIG_VXTEST_BUILD=y {vsb}"
    print("Adding vxtest to VSB...")
    if verbose:
        print(prj_cmd)
    proc = subprocess.run(prj_cmd, capture_output=True, text=True, shell=True)
    if proc.returncode != 0:
        print(proc)
        exit()
